// Just for reference:

// iverilog -o hello hello.vl
// vvp hello

// Network Dispatchable Microprocessor Architecture Test Bench
// NDMA Processor Test Bench

`include "CPU.v"
`include "mem.v"

module TOP;

reg CLK, reset;

// Simulated Clock
initial begin
	CLK <= 0;
	reset <= 1;
	$dumpfile("waves.vcd");
	$dumpvars;
	#20 reset = 0;
end

// toggle clock
always begin
	#5 CLK <= !CLK;
end

// START CPU 1
wire    [31:0]  PC1, Imem1, dAddr1, Din1, Dout1;
wire            dWrite1;
// CPU 1 Network
wire    [31:0]  nBI1, sBI1, wBI1, eBI1;
wire    [31:0]  nBO1, sBO1, wBO1, eBO1;

mem mem1
(
	.CLK	(CLK),
	.MRST	(reset),
	.iAddr	(PC1),
	.Iout	(Imem1),
	.dAddr	(dAddr1),
	.dWrite	(dWrite1),
	.Din	(Din1),
	.Dout	(Dout1)
);
// memory 1 paramaters
defparam mem1.pIMEM_FILE = "memory1.mem";
// end memory 1 parameters

// NDMA CPU
CPU cpu1
(
	.CLK 		(CLK),
	.MRST 		(reset),
	.oIMemAddr	(PC1),
	.iMemIn		(Imem1),
	.dMemIn		(Dout1),
	.dAddr		(dAddr1),
	.dWrite		(dWrite1),
	.WD			(Din1),
    // Network Side (dont need for this test)
	.nBusIn		(nBI1),
	.nBusOut	(nBO1),
	.sBusIn		(nBO3),
	.sBusOut	(sBO1),
	.eBusIn		(wBO2),
	.eBusOut	(eBO1),
	.wBusIn		(wBI1),
	.wBusOut	(wBO1)
);
// END CPU 1

// START CPU 2
wire    [31:0]  PC2, Imem2, dAddr2, Din2, Dout2;
wire            dWrite2;
wire    [31:0]  wIWAddr2, wIData2;
wire            wIWrite2;

// CPU 2 Network
wire    [31:0]  nBI2, sBI2, wBI2, eBI2;
wire    [31:0]  nBO2, sBO2, wBO2, eBO2;

mem mem2
(
	.CLK	(CLK),
	.MRST	(reset),
    // Instruction Memory
	.iAddr	(PC2),
	.Iout	(Imem2),
    .iIData     (wIData2),
    .iIWrite    (wIWrite2),
    .iIWAddr    (wIWAddr2),
    // Data Memory
	.dAddr	(dAddr2),
	.dWrite	(dWrite2),
	.Din	(Din2),
	.Dout	(Dout2)
);
// memory 2 paramaters
defparam mem2.pIMEM_FILE = "memory2.mem";
// end memory 2 parameters

// NDMA CPU 2
CPU cpu2
(
	.CLK 		(CLK),
	.MRST 		(reset),
    // Instruction Memory
	.oIMemAddr	(PC2),
	.iMemIn		(Imem2),
    .oIWAddr    (wIWAddr2),
    .oIWrite    (wIWrite2),
    .oIData     (wIData2),
    // Data Memory
	.dMemIn		(Dout2),
	.dAddr		(dAddr2),
	.dWrite		(dWrite2),
	.WD			(Din2),
    // Network Side (dont need for this test)
	.nBusIn		(nBI2),
	.nBusOut	(nBO2),
	.sBusIn		(nBO4),
	.sBusOut	(sBO2),
	.eBusIn		(eBI2),
	.eBusOut	(eBO2),
	.wBusIn		(eBO1),
	.wBusOut	(wBO2)
);
// END CPU 2

endmodule
