`include "ndmaref.h"

// Takes in two values and operates on them (32 bits)
// opCode is 6 bits
// ALUresult is 32 bits

module ALU
(
    CLK,
    RS_bus,
    RT_bus,
    Immediate,
    useImmediate,
    Operation,
    ALUresult
);

input   wire            CLK;
input   wire    [31:0]  RS_bus;
input   wire    [31:0]  RT_bus;
input   wire    [5:0]   Operation;
input   wire    [31:0]  Immediate;
input   wire            useImmediate;
output  wire    [31:0]  ALUresult;

wire    [31:0]  Toperand;
wire    [4:0]   ShiftAmt;

// Multiply Result
reg     [63:0]  MultResult;

assign Toperand = (useImmediate) ? Immediate : RT_bus;
assign ShiftAmt = (useImmediate) ? Immediate[10:6] : RS_bus[4:0];

// Combinational Based ALU
// Faster sometimes
assign ALUresult = (Operation == `alu_add) ? RS_bus + Toperand :
                   (Operation == `alu_sub) ? RS_bus - Toperand :
                   (Operation == `alu_or)  ? RS_bus | Toperand :
                   (Operation == `alu_and) ? RS_bus & Toperand :
                   (Operation == `alu_xor) ? RS_bus ^ Toperand :
                   (Operation == `alu_nor) ? ~(RS_bus | Toperand) :
                   (Operation == `alu_slt) ? (RS_bus < Toperand) :
                   (Operation == `alu_sltu) ? (RS_bus < Toperand) :  // Need to fix?
                   (Operation == `alu_sll) ? RT_bus << Toperand :
                   (Operation == `alu_srl) ? RT_bus >> Toperand :
                   (Operation == `alu_sra) ? RT_bus >>> Toperand : 32'hzzzzzzzz;
                   /*(Operation == `alu_mult) ? RS_bus * Toperand :
                   (Operation == `alu_multu) ? RS_bus * Toperand :
                   (Operation == `alu_div) ? RS_bus / Toperand :
                   (Operation == `alu_divu) ? RS_bus / Toperand :
                   (Operation == `alu_mod) ? RS_bus % Toperand : 32'hzzzzzzzz; */

endmodule
