`include "ndmaref.h"

// Takes in two values and operates on them (32 bits)
// opCode is 6 bits
// ALUresult is 32 bits

module ALU
(
    CLK,
    RS_bus,
    RT_bus,
    Immediate,
    useImmediate,
    Operation,
    ALUresult
);

input   wire            CLK;
input   wire    [31:0]  RS_bus;
input   wire    [31:0]  RT_bus;
input   wire    [5:0]   Operation;
input   wire    [31:0]  Immediate;
input   wire            useImmediate;
output  reg     [31:0]  ALUresult;

reg     [31:0]  Toperand;
reg     [4:0]   ShiftAmt;

// Multiply Result
reg     [63:0]  MultResult;


always @(RS_bus or RT_bus or Immediate or Operation or useImmediate) begin
	Toperand = (useImmediate) ? Immediate : RT_bus;
	ShiftAmt = (useImmediate) ? Immediate[10:6] : RS_bus[4:0];
	case(Operation)
		`alu_add:	ALUresult <= RS_bus + Toperand;
		`alu_sub:	ALUresult <= RS_bus - Toperand;
		`alu_or	:	ALUresult <= RS_bus | Toperand;
		`alu_and:
		    begin
		        $display("AND! RS:%d Top:%d", RS_bus, Toperand);
                ALUresult <= RS_bus & Toperand;
            end

		`alu_xor:	ALUresult <= RS_bus ^ Toperand;
		`alu_nor:	ALUresult <= ~(RS_bus | Toperand);

		`alu_slt:
		    begin
		        ALUresult <= (RS_bus < Toperand);
                $display("slt: %d < %d rt_bus:%d uI:%d", RS_bus, Toperand, RT_bus, useImmediate);
            end

		`alu_sltu:  $display("sltu not implemented yet!");

		`alu_sll:
		    begin
		        ALUresult <= RT_bus << Toperand;
                $display("sll: %d << %d", RT_bus, Toperand);
            end

		`alu_srl:
            begin
                ALUresult <= RT_bus >> Toperand;
            end

		`alu_sra:
            begin
                ALUresult <= RT_bus >>> Toperand;
            end

        `alu_mult:
            begin
                ALUresult <= RS_bus * Toperand;
                MultResult <= RS_bus * Toperand;
                $display("%d * %d", RS_bus, Toperand);
            end

        `alu_multu:
            begin
                ALUresult <= RS_bus * Toperand;
                MultResult <= RS_bus * Toperand;
                $display("%d * %d", RS_bus, Toperand);
            end

        `alu_div:
            begin
                ALUresult <= RS_bus / Toperand;
                $display("%d / %d", RS_bus, Toperand);
            end

        `alu_divu:
            begin
                ALUresult <= RS_bus / Toperand;
                $display("%d / %d", RS_bus, Toperand);
            end

        `alu_mod:
            begin
                ALUresult <= RS_bus % Toperand;
                $display("%d mod %d", RS_bus, Toperand);
            end

		default:    $display("not implemented!");
	endcase
	//$display("T: %b", RS_bus);
end

endmodule
