// mips like cpu
`include "ndmaref.h"
// Dont Write on this line for some reason

`include "IF.v"
`include "ID.v"
`include "NLayer.v"
`include "RF.v"
`include "ALU.v"
`include "NetCNTL.v"
`include "QC.v"
`include "net_mem_ctrl.v"
`include "in_out.v"

module CPU
(
    CLK, MRST,
    // Instruction Memory
    oIMemAddr, iMemIn,  oIWAddr, oIWrite, oIData,
    // Data Memory
    dMemIn, dAddr, dWrite, WD,
    // Network
    nBusIn, nBusOut,
    sBusIn, sBusOut,
    eBusIn, eBusOut,
    wBusIn, wBusOut,
    // Peripheral Hook Up
    iPortA, iPortB, iPortC, iPortD,
    oPortE, oPortF, oPortG, oPortH
);

input   wire            CLK;
input   wire            MRST;

// Instruction Memory
input   wire    [31:0]  iMemIn;
output  wire    [31:0]  oIMemAddr;
output  wire    [31:0]  oIWAddr;
output  wire    [31:0]  oIData;
output  wire            oIWrite;

assign oIMemAddr = PC;

// Data Memory
input   wire    [31:0]  dMemIn;
///output  reg     [31:0]  WD;
output  wire    [31:0]  WD;
output  wire    [31:0]  dAddr;
output  wire            dWrite;

// Network Layer
input   wire    [31:0]  nBusIn;
input   wire    [31:0]  wBusIn;
input   wire    [31:0]  sBusIn;
input   wire    [31:0]  eBusIn;
output  wire    [31:0]  nBusOut;
output  wire    [31:0]  sBusOut;
output  wire    [31:0]  eBusOut;
output  wire    [31:0]  wBusOut;

// IO Ports
input   wire    [31:0]  iPortA;
input   wire    [31:0]  iPortB;
input   wire    [31:0]  iPortC;
input   wire    [31:0]  iPortD;
output  wire    [31:0]  oPortE;
output  wire    [31:0]  oPortF;
output  wire    [31:0]  oPortG;
output  wire    [31:0]  oPortH;

// Program Counter
reg     [31:0]  PC = 0;
wire    [31:0]  pcInc;
wire    [31:0]  nextPC;
wire            takeBr;
wire    [4:0]   pcSel;
wire    [25:0]  pcJump;
wire    [31:0]  pcJmp;
wire    [31:0]  pcBranch;
wire    [15:0]  pcOffset;

// inputs for rf
wire    [4:0]   RS_reg;
wire    [4:0]   RT_reg;
wire    [4:0]   RD_reg;
wire            rWrite;

wire    [31:0]  RS_bus;
wire    [31:0]  RT_bus;

// ID Vars
wire            wBreak;

// IN OUT
wire    [4:0]   wIOAddr;
wire    [31:0]  wIOWD;
wire    [31:0]  wIOData;
wire            wROWrite;

// ALU variables
wire    [31:0]  ALU_bus;
wire    [31:0]  Imm_bus;
wire    [5:0]   Op_bus;
wire            ImmUse;

wire    [31:0]  Ins;

// Network Layer
wire            dxFlag;
wire    [7:0]   dest_ID;
wire    [7:0]   send_data;
wire            rxFlag;

wire    [7:0]   self_id;
wire            idFlag;
wire    [7:0]   wSelfID;

assign pcInc = 1;
assign nextPC = PC + pcInc;
assign pcJmp = {PC[31:26], pcJump};
assign pcBranch = (pcOffset[15]) ? PC - (~pcOffset + 1) : PC + pcOffset + 1;

always @(pcOffset) begin
    //$display("pcOffset:%d %b", pcOffset, pcOffset);
end

always @(pcSel) begin
    $display("pcJmp: %d pcSel:%d QC: %d", pcJmp, pcSel, QC_out);
end

// increment the clock accordingly
// PC is flopped on negedge because
// otherwise the QC result is not valid
always @(negedge CLK) begin
	if(MRST)
	    begin
	    	PC <= 0;
	    end
	else
	    begin
	    	if(wBreak == 0 && ~wInsLoad)   // This must be removed
                begin
	    	        case(pcSel)
	    	        	`pc_sel_nex:	    PC <= nextPC;
	    	        	`pc_sel_jmp:	    PC <= pcJmp;
	    	        	`pc_sel_jmp_reg:    PC <= RS_bus;
	    	        	`pc_sel_br:		    PC <= pcBranch;
	    	        	default:		    PC <= nextPC;
	    	        endcase
                end
	    end
end

QC qc (
	.CLK	(CLK),
	.MRST	(MRST),
    .Ins    (Ins),
	.A		(RS_bus),
	.B		(RT_bus),
	.QC_out (QC_out)
);


//always @(QC_out) $display("QC_Out: %d A: %d B: %d", QC_out, RS_bus, RT_bus);

// IF *************************************
// Instruction memory contained in IF
// 1 kb (4*256)
IF ifc (
	.CLK		(CLK),
	.MRST		(MRST),
	.PC			(PC),
	.iMemIn		(iMemIn),
	.Iout		(Ins),
	.takeBranch	(takeBr),   // This is deprecated looks like
	.pcSelect	(pcSel),
	.pcJump		(pcJump),
	.pcOffset   (pcOffset),
	.QC			(QC_out)
);

// Debugging Section
// **********************************************************************
always @(Ins) if(!MRST) $display("ID: %d PC:%d, Ins:%b", wSelfID, PC+1, Ins);
//always @(Op_bus)  if(!MRST) $display("PC:%d, Op_bus:%b", PC, Op_bus);
//always @(Imm_bus) if(!MRST) $display("PC:%d, Imm_bus:%d", PC, Imm_bus);
//always @(ImmUse) if(!MRST) $display("PC:%d, ImmUse:%d", PC, ImmUse);
//always @(RS_reg) if(!MRST) $display("PC:%d, RS:%d", PC, RS_reg);
//always @(RT_reg) if(!MRST) $display("PC:%d, RT:%d", PC, RT_reg);
//always @(RD_reg) if(!MRST) $display("PC:%d, RD:%d", PC, RD_reg);
//always @(WD) if(!MRST) $display("PC:%d, WD:%d", PC, WD);

//always @(RS_bus) if(!MRST) $display("PC:%d, RS_bus:%d", PC, RS_bus);

// **********************************************************************

// RD ******************************************
ID id (
	.CLK		(CLK),
	.MRST		(MRST),
	.PC			(PC),
	.Iin		(Ins),
	.OpOut		(Op_bus),
	.Imm		(Imm_bus),
	.uImm		(ImmUse),
	.RS			(RS_reg),
	.RT			(RT_reg),
	.RD			(RD_reg),
	.rW			(rWrite),
    .roW        (wROWrite),
    .dW         (dWrite),
    .oBreak     (wBreak),
    .InsLoad    (wInsLoad)
);

wire            wInsLoad;
wire    [31:0]  wNetIns;

always@(wInsLoad)   begin
    $display("wInsLoad:%b PC:%d", wInsLoad, PC);
end

RF rf
(
	.CLK 		(CLK),
	.MRST 		(MRST),
	.iReadReg1 	(RS_reg),
	.iReadReg2	(RT_reg),
	.iWriteReg	(RD_reg),
	.iWriteData	(WD),
	.iWrite	    (rWrite),
	.oReadData1	(RS_bus),
	.oReadData2	(RT_bus),
    // Network
    .iNetMsg(rxData),
    .iNetMsgWrite(rxFlag),
    .oNetIns(wNetIns),
    .oInsValid(wInsValid)
);

assign wIOAddr = (Ins[`op] == `OUTI) ? RS_reg :
                 (Ins[`op] == `SPECIAL &&
                  Ins[`function] == `IN) ? RS_reg :
                 (Ins[`op] == `SPECIAL &&
                  Ins[`function] == `OUT) ? RD_reg : 5'hzzz;

assign wIOWD = (Ins[`op] == `OUTI) ? Imm_bus : RS_bus;

// IN OUT stuff
in_out io
(
    .iCLK(CLK), .iMRST(MRST),
    // Input
    .iPortA(iPortA), .iPortB(iPortB), .iPortC(iPortC), .iPortD(iPortD),
    // Output
    .oPortE(oPortE), .oPortF(oPortF), .oPortG(oPortG), .oPortH(oPortH),
    // CPU Side
    .iRegAddr(wIOAddr),
    .iRegWD(wIOWD),
    .iRegWrite(wROWrite),
    .oRegData(wIOData)
);

/*always@(Ins or RS_reg or RS_bus or wIOData) begin
    if(Ins[`op] == `OUTI)
        begin
            $display("OUTI RS_reg:%d RS_bus:%d RT_reg: %d RT_bus:%d", RS_reg, RS_bus, RT_reg, RT_bus);
            $display("OUTI wIOAddr:%d oRegData:%d iRegWrite:%b", wIOAddr, wIOData, wROWrite);
        end
end
*/

/*always@(rxData or rxFlag) begin
    $display("rxData:%b, rxFlag:%b", rxData, rxFlag);
end*/

// EX section
ALU alu (
	.CLK			(CLK),
	.RS_bus			(RS_bus),
	.RT_bus			(RT_bus),
	.Immediate		(Imm_bus),
	.useImmediate	(ImmUse),
	.Operation		(Op_bus),
	.ALUresult		(ALU_bus)
);

// MEM Debugging Section
/*
always @(Ins) begin
    if(Ins[`op] == `LW)
        begin
            $display("** LW: PC:%d, ALU_bus:%d WD: %d", PC, ALU_bus, WD);
        end
    if(Ins[`op] == `SW)
        begin
            $display("** SW: PC:%d, ALU_bus:%d WD: %d", PC, ALU_bus, WD);
        end
end
*/

// WB Section  (no data memory currently)
// WD needs to be RT_bus for sw, dMemIn for lw, and ALU_bus for others

// Loads
assign WD = (Ins[`op] == `LW)   ?   dMemIn  :
            (Ins[`op] == `LH)   ?   {{16{dMemIn[15]}}, dMemIn[15:0]} :
            (Ins[`op] == `LHU)  ?   {{16{1'b0}}, dMemIn[15:0]} :
            (Ins[`op] == `LB)   ?   {{24{dMemIn[7]}}, dMemIn[7:0]} :
            (Ins[`op] == `LBU)  ?   {{24{1'b0}}, dMemIn[7:0]} :
            // Stores
            (Ins[`op] == `SW)   ?   RT_bus  :
            (Ins[`op] == `SH)   ?   {{16{1'b0}}, RT_bus[15:0]} :
            (Ins[`op] == `SB)   ?   {{24{1'b0}}, RT_bus[7:0]} :
            // JAL
            (Ins[`op] == `JAL)  ?   PC + 1  :
            // IO
            (Ins[`op] == `SPECIAL &&
            Ins[`function] == `IN)   ?   wIOData : ALU_bus;

/*always@(Ins[`op] or dMemIn or RT_bus or PC or wIOData or ALU_bus) begin
    case(Ins[`op])
        `LW:    WD <= dMemIn;
        `SW:    WD <= RT_bus;
        `JAL:   WD <= PC + 1;
        default: WD <= ALU_bus;
    endcase
end    */

assign dAddr = ALU_bus;

//always @(ALU_bus)  $display("PC:%d, ALU_bus:%d", PC, ALU_bus);

// The net_mem_ctrl is a glorified counter
// simply controls writing of memory instructions to
// the "dual port" instruction memory

net_mem_ctrl nmc
(
    .iCLK(CLK),
    .iMRST(MRST),
    .iNetIns(wNetIns),
    .iNetInsValid(wInsValid),
    .oWAddr(oIWAddr),
    .oWData(oIData),
    .oWrite(oIWrite)
);

NetCNTL net_cnt(
	.CLK		(CLK),
	.Iin		(Ins),
	.Imm		(Imm_bus),
	.RT_in		(RT_bus),
	.dxFlag		(dxFlag),
	.dest_id	(dest_ID),
	.send_data	(send_data),
	.self_id	(self_id),
	.idFlag		(idFlag)
);

always @(dxFlag) $display("dxFlag: %d d_id:%d s_d:%d", dxFlag, dest_ID, send_data);

wire    [7:0]   rxData;

/*
NLayer net_layer (
	.CLK 		(CLK),
	.MRST 		(MRST),
    // Receieve
	.rxFlag 	(rxFlag),
    .rxData     (rxData),
    // Send
	.dxFlag 	(dxFlag),
	.dest_id 	(dest_ID),
	.send_data 	(send_data),
	.nBusIn 	(nBusIn),
	.sBusIn 	(sBusIn),
	.eBusIn 	(eBusIn),
	.wBusIn 	(wBusIn),
	.nBusOut 	(nBusOut),
	.sBusOut 	(sBusOut),
	.eBusOut 	(eBusOut),
	.wBusOut 	(wBusOut),
	.idFlag	 	(idFlag),
	.id		 	(self_id),
    .oSelfID    (wSelfID)
);
*/

//always @(idFlag) $display("idFlag: %d PC: %d", idFlag, PC);

// network debuggin
always @(dxFlag) begin
    if(!MRST && dxFlag == 1)
        begin
            $display("S_Msg id:%d d:%d", dest_ID, send_data);
            //$display("S_Msg_S id:%d d:%d", sBusOut[`dest_id], sBusOut[`msg_data]);
            //$display("S_MSG_S: %b", sBusOut);
        end
end

endmodule
