// Instruction Decoder
`include "ndmaref.h"

module ID
(
    CLK, MRST, PC,
    Iin, OpOut, Imm, uImm,
    RS, RT, RD, rW, dW, roW,
    InsLoad,
    oBreak
);
input CLK;
input MRST;
input [31:0] PC;
input [31:0] Iin;

output reg [5:0] OpOut;
output reg [31:0] Imm;
output reg uImm;			//use Immediate

output reg  [4:0]   RS;
output reg  [4:0]   RT;
output reg  [4:0]   RD;

output  reg         InsLoad;
output  reg         rW;
output  reg         roW;
output  reg         dW;
output  reg         oBreak;// = 0;

always @(posedge CLK) begin
	if(MRST) begin
		//OpOut <= 0;
		//Imm <= 0;
		//uImm <= 0;
		//RS <= 0;
		//RT <= 0;
		//RD <= 0;
		//rW <= 0;
        //dW <= 0;
        //oBreak <= 0;
	end
end


always @(Iin)
begin : REGISTERS
	// RS and RT always from same place (aihgs)
	RS <= Iin[`rs];
	RT <= Iin[`rt];

	// RD reg
	casex ({Iin[`op], Iin[`function]})
		{`SPECIAL, `ADD}:	RD <= Iin[`rd];
		{`SPECIAL, `ADDU}:	RD <= Iin[`rd];
		{`SPECIAL, `SUB}:	RD <= Iin[`rd];
		{`SPECIAL, `SUBU}:	RD <= Iin[`rd];
		{`SPECIAL, `AND}:	RD <= Iin[`rd];
		{`SPECIAL, `OR}:	RD <= Iin[`rd];
		{`SPECIAL, `XOR}:	RD <= Iin[`rd];
		{`SPECIAL, `NOR}:	RD <= Iin[`rd];
		{`SPECIAL, `SLT}:	RD <= Iin[`rd];
		{`SPECIAL, `SLTU}:	RD <= Iin[`rd];
		{`SPECIAL, `SLL}:	RD <= Iin[`rd];
		{`SPECIAL, `SRL}:	RD <= Iin[`rd];
		{`SPECIAL, `SRA}:	RD <= Iin[`rd];
        // Multiply Divide Rem(mod)
        {`SPECIAL, `MULT}:  RD <= Iin[`rd];
        {`SPECIAL, `MULTU}: RD <= Iin[`rd];
        {`SPECIAL, `DIV}:   RD <= Iin[`rd];
        {`SPECIAL, `DIVU}:  RD <= Iin[`rd];
        {`SPECIAL, `MOD}:   RD <= Iin[`rd];
        // IN OUT
        {`SPECIAL, `IN}:    RD <= Iin[`rd];
        {`SPECIAL, `OUT}:   RD <= Iin[`rd];
		// Immediate Instructions
		{`ADDI, `dc6}:		RD <= Iin[`rt];
		{`ADDIU, `dc6}:		RD <= Iin[`rt];
		{`SLTI, `dc6}:		RD <= Iin[`rt];
		{`SLTIU, `dc6}:		RD <= Iin[`rt];
		{`ANDI,  `dc6}:		RD <= Iin[`rt];
		{`ORI,   `dc6}:		RD <= Iin[`rt];
		{`XORI,  `dc6}:		RD <= Iin[`rt];
		{`LUI,	 `dc6}:		RD <= Iin[`rt];
        // JAL
        {`JAL, `dc6}:       RD <= `ra;  // return address
        // Memory Instructions
        {`LW,  `dc6}:       RD <= Iin[`rt];
        {`LH,  `dc6}:       RD <= Iin[`rt];
        {`LHU, `dc6}:       RD <= Iin[`rt];
        {`LB,  `dc6}:       RD <= Iin[`rt];
        {`LBU, `dc6}:       RD <= Iin[`rt];
		// network instructions
		{`SMSG,  `dc6}:		RD <= Iin[`rt];
		{`SID, 	 `dc6}: 	RD <= Iin[`rt];
	endcase
end

// Load Delay
always @(posedge CLK)
begin
    case(Iin[`op])
        `LW:
            begin
                if(InsLoad != 1) InsLoad <= 1;
                else InsLoad <= 0;
            end
        `LH:
            begin
                if(InsLoad != 1) InsLoad <= 1;
                else InsLoad <= 0;
            end
        `LHU:
            begin
                if(InsLoad != 1) InsLoad <= 1;
                else InsLoad <= 0;
            end
        `LB:
            begin
                if(InsLoad != 1) InsLoad <= 1;
                else InsLoad <= 0;
            end
        `LBU:
            begin
                if(InsLoad != 1) InsLoad <= 1;
                else InsLoad <= 0;
            end

        default:    InsLoad <= 0;
    endcase
end

always @(Iin)
begin : OP_CODE
	// OP Code
	casex ({Iin[`op], Iin[`function]})
		{`SPECIAL, `ADD}:	OpOut <= `alu_add;
		{`SPECIAL, `ADDU}:	OpOut <= `alu_add;
		{`SPECIAL, `SUB}:	OpOut <= `alu_sub;
		{`SPECIAL, `SUBU}:	OpOut <= `alu_sub;
		{`SPECIAL, `AND}:	OpOut <= `alu_and;
		{`SPECIAL, `OR}:	OpOut <= `alu_or;
		{`SPECIAL, `XOR}:	OpOut <= `alu_xor;
		{`SPECIAL, `NOR}:	OpOut <= `alu_nor;
		{`SPECIAL, `SLT}:	OpOut <= `alu_slt;
		{`SPECIAL, `SLTU}:	OpOut <= `alu_sltu;
		{`SPECIAL, `SLL}:	OpOut <= `alu_sll;
		{`SPECIAL, `SRL}:	OpOut <= `alu_srl;
		{`SPECIAL, `SRA}:	OpOut <= `alu_sra;
        // Mult Div Mod
        {`SPECIAL, `MULT}:  OpOut <= `alu_mult;
        {`SPECIAL, `MULTU}: OpOut <= `alu_multu;
        {`SPECIAL, `DIV}:   OpOut <= `alu_div;
        {`SPECIAL, `DIVU}:  OpOut <= `alu_divu;
        {`SPECIAL, `MOD}:   OpOut <= `alu_mod;
        // IN OUT
        {`SPECIAL, `IN}:    OpOut <= `alu_add;
        {`SPECIAL, `OUT}:   OpOut <= `alu_add;
        // Memory Ops
        // Loads
        {`LW, `dc6}:        OpOut <= `alu_add;
        {`LH, `dc6}:        OpOut <= `alu_add;
        {`LHU,`dc6}:        OpOut <= `alu_add;
        {`LB, `dc6}:        OpOut <= `alu_add;
        {`LBU,`dc6}:        OpOut <= `alu_add;
        // Stores
        {`SW, `dc6}:        OpOut <= `alu_add;
        {`SH, `dc6}:        OpOut <= `alu_add;
        {`SB, `dc6}:        OpOut <= `alu_add;
		// Immediate Instructions
		{`ADDI, `dc6}:		OpOut <= `alu_add;
		{`ADDIU, `dc6}:		OpOut <= `alu_add;
		{`SLTI, `dc6}:		OpOut <= `alu_slt;
		{`SLTIU, `dc6}:		OpOut <= `alu_sltu;
		{`ANDI,  `dc6}:		OpOut <= `alu_and;
		{`ORI,   `dc6}:		OpOut <= `alu_or;
		{`XORI,  `dc6}:		OpOut <= `alu_xor;
		{`LUI,	 `dc6}:		OpOut <= `alu_or;
        // JAL
        {`JAL,   `dc6}:     OpOut <= `alu_add;
		// network instructions
		{`SMSG,  `dc6}:		OpOut <= `alu_add;
		{`SID, 	 `dc6}:		OpOut <= `alu_add;

	endcase

	// Use Immediate
	uImm <= (	(Iin[`op] == `ADDI)  ||	(Iin[`op] == `ADDIU)  ||
				(Iin[`op] == `SLTI)  || (Iin[`op] == `SLTIU)  ||
				(Iin[`op] == `ANDI)  || (Iin[`op] == `ORI) 	  ||
				(Iin[`op] == `XORI)  || (Iin[`op] == `LUI)	  ||
				(Iin[`op] == `SMSG)	 ||	(Iin[`op] == `SID)	  ||
                // Mem Ops
                // Loads
                (Iin[`op] == `LW)    ||
                (Iin[`op] == `LH)    || (Iin[`op] == `LHU)    ||
                (Iin[`op] == `LB)    || (Iin[`op] == `LBU)    ||
                // Stores
                (Iin[`op] == `SW)    || (Iin[`op] == `SH)     ||
                (Iin[`op] == `SB)    ||
                // Special Ops
				((Iin[`op] == `SPECIAL) && ((Iin[`function] == `SLL) ||
											(Iin[`function] == `SRL) ||
											(Iin[`function] == `SRA) )) );


	// Choose The Proper Immediate
	casex(Iin[`op])
		`ADDI:	Imm <= {{16{Iin[15]}}, {Iin[`immediate]}};
		`ADDIU:	Imm <= {{16{1'b0}}, {Iin[`immediate]}};
		`SLTI:	Imm <= {{16{Iin[15]}}, {Iin[`immediate]}};
		`SLTIU:	Imm <= {{16{Iin[15]}}, {Iin[`immediate]}};

		`ANDI:	Imm <= {16'b0, {Iin[`immediate]}};
		`ORI:	Imm <= {16'b0, {Iin[`immediate]}};
		`XORI:	Imm <= {16'b0, {Iin[`immediate]}};

		// Shift Ops
		`SLL:	Imm <= {27'b0, {Iin[`sa]}};
		`SRL:	Imm <= {27'b0, {Iin[`sa]}};
        `SRA:   Imm <= {27'b0, {Iin[`sa]}};

        // IN OUT
        `OUTI:  Imm <= {{16{Iin[15]}}, {Iin[`immediate]}};

        // Memory Ops
        // Loads
        `LW:    Imm <= {16'b0, Iin[`immediate]};
        `LH:    Imm <= {16'b0, Iin[`immediate]};
        `LHU:   Imm <= {16'b0, Iin[`immediate]};
        `LB:    Imm <= {16'b0, Iin[`immediate]};
        `LBU:   Imm <= {16'b0, Iin[`immediate]};
        // Stores
        `SW:    Imm <= {16'h0, Iin[`immediate]};
        `SH:    Imm <= {16'h0, Iin[`immediate]};
        `SB:    Imm <= {16'h0, Iin[`immediate]};

		`LUI:	Imm <= {{Iin[`immediate]}, 16'b0};

		// Since the message data can only be 8 bits currently we
		// only use the lower 16 bits
		`SMSG:	Imm <= {16'b0, {Iin[`immediate]}};
		`SID:	Imm <= {16'b0, {Iin[`immediate]}};

	endcase
end

// Data Write
always @(Iin)
begin : DATA_WRITE
    if(Iin == 0)
        begin
            dW <= 0;
        end
    else if(Iin[`op] == `SPECIAL)
        begin
            dW <= 0;
        end
    else
        begin
            case(Iin[`op])
                `SW:    dW <= 1;
                `SH:    dW <= 1;
                `SB:    dW <= 1;

                default: dW <= 0;
            endcase
        end
end // end DATA_WRITE

// Write to Register at negative edge of clock (registerWrite)
always @(Iin)
begin : REG_WRITE
	// reg Write
	if(Iin == 0)
	    begin
	        //if(InsLoad) rW <= 1;
            //else rW <= 0;
            rW <= 0;
        end
	else if(Iin[`op] == `SPECIAL)
		begin
		    case (Iin[`function])
		    	`ADD:	rW <= 1;
		    	`ADDU:	rW <= 1;
		    	`SUB:	rW <= 1;
		    	`SUBU:	rW <= 1;
		    	`AND:	rW <= 1;
		    	`OR:	rW <= 1;
		    	`XOR:	rW <= 1;
		    	`NOR:	rW <= 1;
		    	`SLT:	rW <= 1;
		    	`SLTU:	rW <= 1;
		    	`SLL:   rW <= 1;
		    	`SRL:	rW <= 1;
		    	`SRA:	rW <= 1;
                // mult div mod
                `MULT:  rW <= 1;
                `MULTU: rW <= 1;
                `DIV:   rW <= 1;
                `DIVU:  rW <= 1;
                `MOD:   rW <= 1;
                // IN OUT
                `IN:    rW <= 1;
                `OUT:   rW <= 0;
		    	default:			rW <= 0;
		    endcase
		end
	else
		begin
		    case (Iin[`op])
		    	// immediate instructions
		    	`ADDI:		rW <= 1;
		    	`ADDIU:		rW <= 1;
		    	`SLTI:		rW <= 1;
		    	`SLTIU:		rW <= 1;
		    	`ANDI:		rW <= 1;
		    	`ORI:		rW <= 1;
		    	`XORI:		rW <= 1;
		    	`LUI:		rW <= 1;
                // JAL
                `JAL:       rW <= 1;
                // Memory Ops
                // Loads
                `LW:        rW <= 1;
                `LH:        rW <= 1;
                `LHU:       rW <= 1;
                `LB:        rW <= 1;
                `LBU:       rW <= 1;
                // Stores
                `SW:        rW <= 0;
                `SH:        rW <= 0;
                `SB:        rW <= 0;

                // IN OUT
                `OUTI:      rW <= 0;

		    	// network instructions
		    	`SMSG:		rW <= 0;
		    	`SID:		rW <= 0;

		    	default:	rW <= 0;
		    endcase
		end
end

// IN OUT Register Write
always@(Iin)
begin : INOUT_WRITE
    if(Iin == 0)
	    begin
	        roW <= 0;
        end
	else if(Iin[`op] == `SPECIAL)
		begin
		    case (Iin[`function])
                `OUT:       roW <= 1;
		        default:    roW <= 0;
		    endcase
		end
	else
		begin
		    case (Iin[`op])
                `OUTI:      roW <= 1;
		    	default:	roW <= 0;
		    endcase
		end
end

//always @(rW) $display("rW change!");


// break instruction
always @(Iin)
begin : BREAK_KILL
    if(MRST)
        begin
            oBreak <= 0;
        end
    else
        begin
	        casex({Iin[`op], Iin[`function]})
	        	{`SPECIAL, `BREAK}:
	        	    begin
                        $display("Break Encountered");
                        // Need to halt PC some how
                        oBreak <= 1;
	        	        $stop;
                    end
                default: oBreak <= 0;
	        endcase
        end
end


endmodule
