// Instruction Fetch Module

module IF(CLK, MRST, PC, iMemIn, QC, Iout, takeBranch, pcSelect, pcJump, pcOffset);
input CLK;
input MRST;
input [31:0] PC;
input wire [31:0] iMemIn;
input QC;

output wire [31:0] Iout;
output reg takeBranch;
output reg [4:0] pcSelect;
output reg [25:0] pcJump;
output wire[15:0] pcOffset;

integer i;

// The idea is to have "on chip memory"
// This may need to be changed however
// 1 Kb of memory (32*256/8)
// going to use verilog 1995 style
reg [31:0] mem [255:0]	/* synthesis ramstyle = "M4K" */;

assign Iout = iMemIn;

assign pcOffset = Iout[`immediate];

// Loads memory from file
initial begin
	//if(iMem) begin
			//$readmemb("imemory.mem", mem);
			//$display("memory initialized");
	//end else for(i=0;i<32;i=i+1) mem[i] <= 0;
	takeBranch = 0;
	pcSelect = 0;
	pcJump = 0;
end


// take branch
always @(Iout) begin
    if(Iout[`op] == `SPECIAL)
        begin
            case(Iout[`function])
                `JR:
                    begin
                        takeBranch <= 1;
                    end
            endcase
        end
    else
        begin
	        case(Iout[`op])
	        	`J:			takeBranch <= 1;
                `JAL:       takeBranch <= 1;
	        	`BEQ:		begin
	        					//takeBranch <= 1;
	        					//$display("branch_E! QC: %d", QC);
	        				end
	        	`BNE:		begin
	        					takeBranch <= 1;
	        					//$display("branch_N! QC: %d", QC);
	        				end
	        	default:	takeBranch <= 0;
	        endcase
        end
end

// pcSelect
always @(QC or Iout) begin
    if(Iout[`op] == `SPECIAL)
        begin
            case(Iout[`function])
                `JR:
	        	    begin
	        	        pcSelect <= `pc_sel_jmp_reg;
                        $display("JR!");
                    end

                default:
	        	    begin
	        	        pcSelect <= `pc_sel_nex;
                    end
            endcase
        end
    else
        begin
	        casex({Iout[`op], QC})
	        	{`J, `dc}:
	        	    begin
	        	        pcSelect <= `pc_sel_jmp;
                    end
                {`JAL, `dc}:
                    begin
                        pcSelect <= `pc_sel_jmp;
                    end

	        	{`BEQ, `true}:		pcSelect <= `pc_sel_br;
	        	{`BNE, `false}:		pcSelect <= `pc_sel_br;
                {`BGTZ, `true}:
                    begin
                        pcSelect <= `pc_sel_br;
                    end

                {`BGEZ, `true}:
                    begin
                        pcSelect <= `pc_sel_br;
                    end

                {`BLTZ, `true}:
                    begin
                        pcSelect <= `pc_sel_br;
                    end

                {`BLEZ, `true}:
                    begin
                        pcSelect <= `pc_sel_br;
                    end

	        	default:
	        	    begin
	        	        pcSelect <= `pc_sel_nex;
                    end
	        endcase
        end
end

// pcJump
always @(Iout) begin
	case(Iout[`op])
		`J:			pcJump <= Iout[`target];
        `JAL:       pcJump <= Iout[`target];
	endcase
end



endmodule
