// Register File
// 32 General Purpose Registers

module RF
(
    CLK, MRST,
    // Register File
    iReadReg1,
    iReadReg2,
    iWriteReg,
    iWriteData,
    iWrite,
    oReadData1,
    oReadData2,

    // Network RF
    iNetMsg,
    iNetMsgWrite,
    oNetIns,
    oInsValid
);

input   wire            CLK;
input   wire            MRST;

// Register File
input   wire    [4:0]   iReadReg1;
input   wire    [4:0]   iReadReg2;
input   wire    [4:0]   iWriteReg;
input   wire    [31:0]  iWriteData;
input   wire            iWrite;
output  wire    [31:0]  oReadData1;
output  wire    [31:0]  oReadData2;

// Network RF
input   wire    [7:0]   iNetMsg;
input   wire            iNetMsgWrite;
output  wire    [31:0]  oNetIns;
output  reg             oInsValid;

integer i;

// 32 multipurpose registers
reg     [31:0]  registerFile    [0:31];

// Network Purpose Registers
reg     [7:0]   rInsBuf     [0:3];
reg     [1:0]   rInsBufCnt;

assign oNetIns = {rInsBuf[3], rInsBuf[2], rInsBuf[1], rInsBuf[0]};

assign oReadData1 = registerFile[iReadReg1];
assign oReadData2 = registerFile[iReadReg2];

// We write to the register file on the negative edge of the clock
// This means all of the stages must complete in that time.
always @(negedge CLK)
begin : WRITE_DATA
	if(MRST)
		begin
			for(i = 0; i < 32; i = i + 1)
			    begin
			        registerFile[i] <= 0;
			    end
            for(i = 0; i < 4; i = i + 1)
                begin
                    rInsBuf[i] <= 0;
                end
            rInsBufCnt <= 0;
            oInsValid <= 0;
		end
	else
		begin
		    // Network Instruction Buffer
            if(iNetMsgWrite)
                begin
                    rInsBuf[rInsBufCnt] <= iNetMsg;
                    if(rInsBufCnt < 3)
                        begin
                            rInsBufCnt <= rInsBufCnt + 1;
                        end
                    else
                        begin
                            rInsBufCnt <= 0;
                            oInsValid <= 1;
                        end
                end
            else
                begin
                    if(oInsValid)
                        begin
                            $display("Ins Valid:%b", oNetIns);
                            for(i = 0; i < 4; i = i + 1) rInsBuf[i] <= 8'hzz;
                        end
                    oInsValid <= 0;
                end

			// Register File
			if(iWrite)
			    begin
			        if(iWriteReg != 0)
			            begin
			                registerFile[iWriteReg] = iWriteData;
			                $display("RF: Register $%d value: %h", iWriteReg, registerFile[iWriteReg]);
			            end
                    else
                        begin
                            $display("Invalid Attempted Write on $0");
                        end
			    end
		end
end

endmodule
