// Just for reference:

// iverilog -o hello hello.vl
// vvp hello

// Network Dispatchable Microprocessor Architecture Test Bench
// NDMA Processor Test Bench

`include "CPU.v"
`include "mem.v"

module TOP;

reg CLK, reset;

// Simulated Clock
initial begin
	CLK <= 0;
	reset <= 1;
	$dumpfile("waves.vcd");
	$dumpvars;
	#20 reset = 0;
end

// Dug Break!
always begin
    #500 begin
        $display("Break Time!");
        $stop;
    end
end

// toggle clock
always begin
	#5 CLK <= !CLK;
end

// START CPU 1
wire    [31:0]  PC1, Imem1, dAddr1, Din1, Dout1;
wire            dWrite1;
// CPU 1 Network
wire    [31:0]  nBI1, sBI1, wBI1, eBI1;
wire    [31:0]  nBO1, sBO1, wBO1, eBO1;

mem mem1
(
	.CLK	(CLK),
	.MRST	(reset),
	.iAddr	(PC1),
	.Iout	(Imem1),
	.dAddr	(dAddr1),
	.dWrite	(dWrite1),
	.Din	(Din1),
	.Dout	(Dout1)
);
// memory 1 paramaters
defparam mem1.pIMEM_FILE = "memory.mem";
// end memory 1 parameters

// NDMA CPU
CPU cpu1
(
	.CLK 		(CLK),
	.MRST 		(reset),
	.oIMemAddr	(PC1),
	.iMemIn		(Imem1),
	.dMemIn		(Dout1),
	.dAddr		(dAddr1),
	.dWrite		(dWrite1),
	.WD			(Din1),
    // Network Side (dont need for this test)
	.nBusIn		(nBI1), .nBusOut	(nBO1),
	.sBusIn		(nBO3), .sBusOut	(sBO1),
	.eBusIn		(wBO2), .eBusOut	(eBO1),
	.wBusIn		(nBI1), .wBusOut	(wBO1),
    // IO Side
    .iPortA(1),
    .iPortB(10'b0101010101)
);
// END CPU 1

endmodule
