// Input output ports stay within CPU conventions
// of 32 bit width

module in_out
(
    iCLK, iMRST,
    // Input
    iPortA, iPortB, iPortC, iPortD,
    // Output
    oPortE, oPortF, oPortG, oPortH,
    // CPU Side
    iRegAddr,
    iRegWD,
    iRegWrite,
    oRegData,
);

input   wire            iCLK;
input   wire            iMRST;

// Input
input   wire    [31:0]  iPortA; // Addressed as $1
input   wire    [31:0]  iPortB; // Addressed as $2
input   wire    [31:0]  iPortC; // Addressed as $3
input   wire    [31:0]  iPortD; // Addressed as $4

// Output
output  reg     [31:0]  oPortE; // Addressed as $5
output  reg     [31:0]  oPortF; // Addressed as $6
output  reg     [31:0]  oPortG; // Addressed as $7
output  reg     [31:0]  oPortH; // Addressed as $8

input   wire    [4:0]   iRegAddr;
input   wire    [31:0]  iRegWD;
input   wire            iRegWrite;
output  wire    [31:0]  oRegData;

// Output Data Mux
assign oRegData = (iRegWrite) ? 32'hzzzzzzzz :
                  (iRegAddr == `r1) ? iPortA :
                  (iRegAddr == `r2) ? iPortB :
                  (iRegAddr == `r3) ? iPortC :
                  (iRegAddr == `r4) ? iPortD :
                  (iRegAddr == `r5) ? oPortE :
                  (iRegAddr == `r6) ? oPortF :
                  (iRegAddr == `r7) ? oPortG :
                  (iRegAddr == `r8) ? oPortH : 32'hxxxxxxxx;

// Use Negative Edge just like RF
always@(negedge iCLK) begin
    if(iMRST)
        begin
            oPortE <= 0;
            oPortF <= 0;
            oPortG <= 0;
            oPortH <= 0;
        end
    else
        begin
            if(iRegWrite)
                begin
                    if(iRegAddr >= `r5 && iRegAddr <= `r8)
                        begin
                            case(iRegAddr)
                                `r5: oPortE <= iRegWD;
                                `r6: oPortF <= iRegWD;
                                `r7: oPortG <= iRegWD;
                                `r8: oPortH <= iRegWD;
                            endcase
                            $display("** INOUT wrote %h to %d", iRegWD, iRegAddr);
                        end
                    else
                        begin
                            $display("INOUT attempted illegal write to r%d", iRegAddr);
                        end

                end
        end
end


endmodule
