`include "CPU.v"
`include "mem.v"

module TOP;
reg CLK, reset;

wire [31:0] nBI1;
wire [31:0] sBI1;
wire [31:0] wBI1;
wire [31:0] eBI1;
wire [31:0] nBO1;
wire [31:0] sBO1;
wire [31:0] wBO1;
wire [31:0] eBO1;
wire [31:0] PC1;
wire [31:0] Imem1;
wire [31:0] dAddr1;
wire dWrite1;
wire [31:0] Din1;
wire [31:0] Dout1;


wire [31:0] nBI2;
wire [31:0] sBI2;
wire [31:0] wBI2;
wire [31:0] eBI2;
wire [31:0] nBO2;
wire [31:0] sBO2;
wire [31:0] wBO2;
wire [31:0] eBO2;
wire [31:0] PC2;
wire [31:0] Imem2;

wire [31:0] nBI3;
wire [31:0] sBI3;
wire [31:0] wBI3;
wire [31:0] eBI3;
wire [31:0] nBO3;
wire [31:0] sBO3;
wire [31:0] wBO3;
wire [31:0] eBO3;
wire [31:0] PC3;
wire [31:0] Imem3;

wire [31:0] nBI4;
wire [31:0] sBI4;
wire [31:0] wBI4;
wire [31:0] eBI4;
wire [31:0] nBO4;
wire [31:0] sBO4;
wire [31:0] wBO4;
wire [31:0] eBO4;
wire [31:0] PC4;
wire [31:0] Imem4;

initial begin
	CLK <= 0;
	reset <= 1;
	$dumpfile("waves.vcd");
	$dumpvars;
	
	#20 reset = 0;
	
end

// toggle clock
always begin
	#5 CLK <= !CLK;
end

/* Instantiate the CPU net
* 9 nodal net
*    ||   ||   	  ||
*  = N1 = N2  	= N3 =
*    ||   ||  	  ||
*  = N3 = N4 	= N6 =
*    ||   ||      ||
*
*  = N7 = N8  = N9 =
*    ||   ||    ||
* Connected as shown above
*********************************/

mem mem1 (
	.CLK	(CLK),
	.MRST	(reset),
	.iAddr	(PC1),
	.iMem	(1),
	.Iout	(Imem1),
	.dAddr	(dAddr1), 
	.dWrite	(dWrite1), 
	.Din	(Din1), 
	.Dout	(Dout1)
);

CPU cpu1 (
	.CLK 		(CLK),
	.MRST 		(reset),
	.PC	  		(PC1),
	.iMemIn		(Imem1),
	.dMemIn		(Dout1), 
	.dAddr		(dAddr1), 
	.dWrite		(dWrite1), 
	.WD			(Din1),
	.nBusIn		(nBI1), 
	.nBusOut	(nBO1),	
	.sBusIn		(nBO3), 
	.sBusOut	(sBO1),
	.eBusIn		(wBO2), 
	.eBusOut	(eBO1),
	.wBusIn		(nBI1), 
	.wBusOut	(wBO1)		
);
/*

mem mem2 (
	.CLK	(CLK),
	.MRST	(reset),
	.iAddr	(PC2),
	.iMem	(2),
	.Iout	(Imem2)
);

CPU cpu2 (
	.CLK 		(CLK),
	.MRST 		(reset),
	.PC	  		(PC2),
	.iMemIn		(Imem2),
	.nBusIn		(nBI2), 
	.nBusOut	(nBO2),	
	.sBusIn		(nBO4), 
	.sBusOut	(sBO2),
	.eBusIn		(eBI2), 
	.eBusOut	(eBO2),
	.wBusIn		(eBO1), 
	.wBusOut	(wBO2)		
);

mem mem3 (
	.CLK	(CLK),
	.MRST	(reset),
	.iAddr	(PC3),
	.iMem	(3),
	.Iout	(Imem3)
);

CPU cpu3 (
	.CLK 		(CLK),
	.MRST 		(reset),
	.PC	  		(PC3),
	.iMemIn		(Imem3),
	.nBusIn		(sBO1), 
	.nBusOut	(nBO3),	
	.sBusIn		(sBI3), 
	.sBusOut	(sBO3),
	.eBusIn		(wBO4), 
	.eBusOut	(eBO3),
	.wBusIn		(nBI3), 
	.wBusOut	(wBO3)		
);

mem mem4 (
	.CLK	(CLK),
	.MRST	(reset),
	.iAddr	(PC4),
	.iMem	(4),
	.Iout	(Imem4)
);

CPU cpu4 (
	.CLK 		(CLK),
	.MRST 		(reset),
	.PC	  		(PC4),
	.iMemIn		(Imem4),
	.nBusIn		(sBO2), 
	.nBusOut	(nBO4),	
	.sBusIn		(sBI4), 
	.sBusOut	(sBO4),
	.eBusIn		(wBO3), 
	.eBusOut	(eBO4),
	.wBusIn		(wBI3), 
	.wBusOut	(wBO4)
);


*/


endmodule
