// This interprets information incoming from the PS2 port. It will
// output the scan code.  If the process needs to have the full PS2
// data flow this can be done with the second usage template

`define PS2_INTERFACE_DE2 .iPS2_CLK(PS2_CLK), .iPS2_DAT(PS2_DAT)

// Usage Template (scan code only)
//PS2 ps2(
//    `PS2_INTERFACE_DE2,
//    .iMRST( ),
//    .oKeyValid( ),
//    .oScancode( ),
//    .oShiftEnable( )
//);
//
// Usage Template (full packet stream)
//PS2 ps2(
//    `PS2_INTERFACE_DE2,
//    .iMRST( ),
//    .oKeyBusy( ), oKeyValid( ),
//    .oPS2_DATA( ),
//    .oShiftEnable( )
//);

module PS2(
    iPS2_CLK, iPS2_DAT,
    iMRST,
    oKeyBusy, oKeyValid,
    oScancode,
    //oPS2_DATA,
    oShiftEnable
);
// Inputs
input   wire            iPS2_CLK;
input   wire            iPS2_DAT;
input   wire            iMRST;

// Outputs
// These two outputs verify the state of the PS2
output  wire            oKeyBusy;
output  reg             oKeyValid = 0;

output  reg     [7:0]   oScancode;

reg     [10:0]  oPS2_DATA   [0:5];

output  reg             oShiftEnable = 0;

// State Registers
reg [3:0] rcPS2Bit = 0;
reg [3:0] rcPS2Packet = 0;
reg rfBreakCode = 0;

assign oKeyBusy = (rcPS2Bit != 0) && (iPS2_DAT != 0);

always @(negedge iPS2_CLK) begin
    if(iMRST)
        begin
            rcPS2Packet <= 0;
            rcPS2Bit <= 0;
            rfBreakCode <= 0;
            oShiftEnable <= 0;
            oScancode <= 0;
            oKeyValid <= 0;
        end
    else
        begin
	        if(rcPS2Bit == 0 && iPS2_DAT == 0)
	        	begin
	        	    oKeyValid <= 0;
	        	    oPS2_DATA[rcPS2Packet][0] <= 0;
	        	    rcPS2Bit <= rcPS2Bit + 1;
	        	end
	        else if(rcPS2Bit > 0 && rcPS2Bit < 10)
	        	begin
	        	    oPS2_DATA[rcPS2Packet][rcPS2Bit] <= iPS2_DAT;
	        	    rcPS2Bit <= rcPS2Bit + 1;
	        	end
	        else if(rcPS2Bit == 10)
	        	begin

	        	    oPS2_DATA[rcPS2Packet][rcPS2Bit] <= iPS2_DAT;
	        	    rcPS2Bit <= 0;

	        	    if(	oPS2_DATA[rcPS2Packet][8:1] == 8'h12 ||
	        	    	oPS2_DATA[rcPS2Packet][8:1] == 8'h59)
	        	    	begin
	        	    	    oShiftEnable <= 1;
                        end

	        	    if(oPS2_DATA[rcPS2Packet][8:1] == 8'hF0 && rfBreakCode == 0)
	        	    	begin
	        	    	    rfBreakCode <= 1;
	        	    	    rcPS2Packet <= rcPS2Packet + 1;
	        	    	end
	        	    else if(rfBreakCode == 1)
	        	    	begin
	        	    	    if(	oPS2_DATA[rcPS2Packet][8:1] == 8'h12 ||
	        	    	        oPS2_DATA[rcPS2Packet][8:1] == 8'h59)
	        	    	        begin
	        	    	            oShiftEnable <= 0;
                                    oScancode <= oPS2_DATA[rcPS2Packet][8:1];
                                    rcPS2Packet <= 0;
                                    rfBreakCode <= 0;
                                    oKeyValid <= 0;     // shift is not "valid"
	        	    	        end
                            else
                                begin
	        	    	            // a specific key has been released
	        	    	            oScancode <= oPS2_DATA[rcPS2Packet][8:1];
	        	    	            rcPS2Packet <= 0;
	        	    	            rfBreakCode <= 0;
                                    oKeyValid <= 1;
                                end
	        	    	end
	        	    else rcPS2Packet <= rcPS2Packet + 1;
	        	end
        end // end else if
end     // end always@(negedge PS2_CLK)

endmodule
