// This module will take input from the keyboard on the PS2 port
// and push it's equivalent ASCII value into a small block of m4k memory.
// This uses an internal dedicated buffer but as you can tell it would
// be not too hard to alter to use an external buffer.  When the PS2 key
// input is not valid the module will output the addressed point in memory

// usage template

`include "./PS2.v"

module PS2Keyboard(
    iCLK,
    iPS2_CLK, iPS2_DAT
    iMRST,
    // MEMORY HOOKUP
    iMemAddr, oReadData
);
// Inputs
input   wire            iCLK;
input   wire            iPS2_CLK;
input   wire            iPS2_DAT;
input   wire            iMRST;
input   wire    [7:0]   iMemAddr;
// Outputs
output  wire    [7:0]   oReadData;

// Internal Registers
reg     [7:0]   rpMemAddr;

// Internal Wires
wire    [7:0]   wASCII;
wire            wKeyValid;
wire    [7:0]   wScancode;
wire            wShiftEn;
wire    [7:0]   wMemAddr;

assign wMemAddr = (oKeyValid) ? rpMemAddr : iMemAddr;

M4KMEM m4kmem(
    .iCLK(iCLK), .iMRST(iMRST),
    .iWE_N(~wKeyValid), .iAddr(wMemAddr),
    .iWriteData(wASCII), .oReadData(oReadData)
);
// m4kmem parameters
defparam m4kmem.pRAM_ADDR_WIDTH = 8;
         m4kmem.pRAM_UNIT_COUNT = 256;
         m4kmem.pRAM_UNIT_WIDTH = 8;
// end m4kmem parameters

ScanCodeDecoder scd(
	.iScancode(wScancode),
    .iShiftEn(wShiftEn),
	.oCharOut(wASCII)
);

PS2 ps2(
    .iPS2_CLK(iPS2_CLK), .iPS2_DAT(iPS2_DAT),
    .iMRST(iMRST),
    .oKeyValid(wKeyValid),
    .oScancode(wScancode),
    .ofShiftEnable(wShiftEn)
);

endmodule
