// LCD Display Controller.  This will display the contents of memory in
// standard ASCII format on the LCD display.  Start pointer sets the
// offset from the begining of memory that will be displayed. This is
// designed for a 8x256 memory.  Other sizes will not address correctly
// and would need some size adjustments below.

`define LCD_INTERFACE_DE2 .oLCD_ON(LCD_ON),.oLCD_BLON(LCD_BLON),.oLCD_EN(LCD_EN),.oLCD_RW(LCD_RW),.oLCD_RS(LCD_RS),.oLCD_DATA(LCD_DATA)

// Usage Template
//LCDCNT myLCD(
//	.iCLK (CLOCK_50),
//	.iMRST( ),
//  `LCD_INTERFACE_DE2,
//	.oAddr( 8b ),
//	.iInputData( 8b ),
//	.iSP( 8b )
//);

module LCDCNT (
	iCLK, iMRST,
	oLCD_ON, oLCD_BLON, oLCD_EN,
	oLCD_RW, oLCD_RS, oLCD_DATA,
	oAddr,
	iInputData,
	iSP
);
input iCLK;
input iMRST;

// input should be one 8 bit char
input [7:0] iInputData;
input wire [7:0] iSP;

output wire [7:0] oAddr;

// LCD Setup
output  wire            oLCD_ON      = 1'b1;
output  wire            oLCD_BLON    = 1'b1;
output  wire            oLCD_EN;
output  reg             oLCD_RW      = 0;
output  reg             oLCD_RS      = 0;
output  reg     [7:0]   oLCD_DATA    = 8'b00000001;

reg [7:0] rcLCDPosition = 0;

// Slow down the frequency so that LCD
// registers changes on the lines
reg [12:0]  rcLCD_CLK = 0;
reg         rLCD_CLK = 0;

reg [7:0] rcMemoryPosition = 0;

always @(posedge iCLK) begin
	rcLCD_CLK = rcLCD_CLK + 1;

	if(rcLCD_CLK == 2500) begin
		rLCD_CLK = ~rLCD_CLK;
		rcLCD_CLK = 0;
	end
end

assign oLCD_EN = ~rLCD_CLK;

// we wait a period of ~40us for each command
// LCD Controller State Machine
always @(posedge rLCD_CLK) begin
	if(rcLCDPosition == 0)	begin
		oLCD_DATA <= 8'b00000001;
		oLCD_RW <= 0;
		oLCD_RS <= 0;
		rcLCDPosition <= rcLCDPosition + 1;
	end	else if(rcLCDPosition == 1) begin
		oLCD_RW <= 0;
		oLCD_RS <= 0;
		oLCD_DATA <= 8'b10000000;
		rcLCDPosition <= rcLCDPosition + 1;
		rcMemoryPosition <= 0;
	end else if(rcLCDPosition >= 2 && rcLCDPosition < 18) begin
		oLCD_RS <= 1;
		oLCD_RW <= 0;
		oLCD_DATA <= iInputData[7:0];
		rcLCDPosition <= rcLCDPosition + 1;
		rcMemoryPosition <= rcMemoryPosition + 1;
	end else if(rcLCDPosition == 18) begin
		oLCD_RW <= 0;
		oLCD_RS <= 0;
		oLCD_DATA <= 8'b11000000;
		rcLCDPosition <= rcLCDPosition + 1;
	end else if(rcLCDPosition >= 19 && rcLCDPosition < 35) begin
		oLCD_RS <= 1;
		oLCD_RW <= 0;
		oLCD_DATA <= iInputData[7:0];

		if (rcLCDPosition == 34) begin
			rcLCDPosition <= 1;
			rcMemoryPosition <= 0;
		end
		else begin
			rcLCDPosition <= rcLCDPosition + 1;
			rcMemoryPosition <= rcMemoryPosition + 1;
		end
	end
end

assign oAddr = rcMemoryPosition + (iSP << 4);

endmodule
