// Usage Template:

//M4KMEM m4kmem(
//    .iCLK( ), .iMRST( ),
//    .iWE_N( ), .iAddr( ),
//    .iWriteData( ), .oReadData( )
//);
//// m4kmem parameters
//defparam m4kmem.pRAM_ADDR_WIDTH = 8,
//         m4kmem.pRAM_UNIT_COUNT = 256,
//         m4kmem.pRAM_UNIT_WIDTH = 32;
//// end m4kmem parameters

module M4KMEM(
    iCLK, iMRST,
    iWE_N, iAddr,
    iWriteData,
    oReadData
);

parameter pRAM_ADDR_WIDTH = 8;
parameter pRAM_UNIT_COUNT = 256;
parameter pRAM_UNIT_WIDTH = 32;

parameter pNO_PLI_INIT_FILE = "mem.rif";
parameter pINIT_FILE = "mem.mif";

input wire iCLK;
input wire iMRST;
input wire iWE_N;
input wire [(pRAM_ADDR_WIDTH - 1):0] iAddr;
input wire [(pRAM_UNIT_WIDTH - 1):0] iWriteData;
output wire [(pRAM_UNIT_WIDTH - 1):0] oReadData;

altsyncram	altsyncram_component
(
    .wren_a (~iWE_N),
    .clock0 (iCLK),
    .clock1 (1'b1),
    .address_a (iAddr),
    .address_b (1'b1),
    .data_a (iWriteData),
    .data_b (1'b1),
    .q_a (oReadData),
     .q_b (),
    .aclr0 (1'b0),
    .aclr1 (1'b0),
    .addressstall_a (1'b0),
    .addressstall_b (1'b0),
    .byteena_a (1'b1),
    .byteena_b (1'b1),
    .clocken0 (1'b1),
    .clocken1 (1'b1),
    .rden_b (1'b1),
    .wren_b (1'b0)
);

// altsyncram parameters
defparam
		altsyncram_component.clock_enable_input_a = "BYPASS",
		altsyncram_component.clock_enable_output_a = "BYPASS",
`ifdef NO_PLI
		altsyncram_component.init_file = pNO_PLI_INIT_FILE
`else
		altsyncram_component.init_file = pINIT_FILE
`endif,
		altsyncram_component.intended_device_family = "Cyclone II",
		altsyncram_component.lpm_hint = "ENABLE_RUNTIME_MOD=NO",
		altsyncram_component.lpm_type = "altsyncram",
		altsyncram_component.numwords_a = pRAM_UNIT_COUNT,
		altsyncram_component.operation_mode = "SINGLE_PORT",
		altsyncram_component.outdata_aclr_a = "NONE",
		altsyncram_component.outdata_reg_a = "UNREGISTERED",
		altsyncram_component.power_up_uninitialized = "FALSE",
		altsyncram_component.ram_block_type = "M4K",
		altsyncram_component.widthad_a = pRAM_ADDR_WIDTH,
		altsyncram_component.width_a = pRAM_UNIT_WIDTH,
		altsyncram_component.width_byteena_a = 1;

endmodule
