// Unlike the normal MK4MEM module MK4MEM_DUALPORT allows synchronous
// reads and writes from two independent points in memory.  In the case
// that a read is done to the same address as a write is done the outcome
// is not known.

// Usage Template:

//M4KMEM_DUALPORT m4kmem(
//    .iCLK( ), .iMRST( ),
//    .iWE_N( ),
//    .iWriteAddr( ), .iWriteData( ),
//    .iReadAddr(), .oReadData( )
//);
//// m4kmem parameters
//defparam m4kmem.pRAM_ADDR_WIDTH = 8,
//         m4kmem.pRAM_UNIT_COUNT = 256,
//         m4kmem.pRAM_UNIT_WIDTH = 32;
//// end m4kmem parameters

module M4KMEM_DUALPORT(
    iCLK, iMRST,
    // Write
    iWE_N,
    iWriteAddr,
    iWriteData,
    // Read
    iReadAddr,
    oReadData
);

parameter pRAM_ADDR_WIDTH = 8;
parameter pRAM_UNIT_COUNT = 256;
parameter pRAM_UNIT_WIDTH = 32;

parameter pNO_PLI_INIT_FILE = "mem.rif";
parameter pINIT_FILE = "mem.mif";

input   wire            iCLK;
input   wire            iMRST;
// Write
input   wire                                iWE_N;
input   wire    [(pRAM_ADDR_WIDTH - 1):0]   iWriteAddr;
input   wire    [(pRAM_UNIT_WIDTH - 1):0]   iWriteData;
// Read
input   wire    [(pRAM_ADDR_WIDTH - 1):0]   iReadAddr;
output  wire    [(pRAM_UNIT_WIDTH - 1):0]   oReadData;

altsyncram	altsyncram_component
(
    .wren_a (~iWE_N),
	.clock0 (iCLK),
	.address_a (iWriteAddr),
	.address_b (iReadAddr),
	.data_a (iWriteData),
	.q_b (oReadData),
	.aclr0 (1'b0),
	.aclr1 (1'b0),
	.addressstall_a (1'b0),
	.addressstall_b (1'b0),
	.byteena_a (1'b1),
	.byteena_b (1'b1),
	.clock1 (1'b1),
	.clocken0 (1'b1),
	.clocken1 (1'b1),
	.clocken2 (1'b1),
	.clocken3 (1'b1),
	.data_b ({8{1'b1}}),
	.eccstatus (),
	.q_a (),
	.rden_a (1'b1),
	.rden_b (1'b1),
	.wren_b (1'b0)
);

// altsyncram parameters
defparam
		altsyncram_component.address_reg_b = "CLOCK0",
		altsyncram_component.clock_enable_input_a = "BYPASS",
		altsyncram_component.clock_enable_input_b = "BYPASS",
		altsyncram_component.clock_enable_output_a = "BYPASS",
		altsyncram_component.clock_enable_output_b = "BYPASS",
		altsyncram_component.init_file = pINIT_FILE,
		altsyncram_component.intended_device_family = "Cyclone II",
		altsyncram_component.lpm_type = "altsyncram",
		altsyncram_component.numwords_a = pRAM_UNIT_COUNT,
		altsyncram_component.numwords_b = pRAM_UNIT_COUNT,
		altsyncram_component.operation_mode = "DUAL_PORT",
		altsyncram_component.outdata_aclr_b = "NONE",
		altsyncram_component.outdata_reg_b = "CLOCK0",
		altsyncram_component.power_up_uninitialized = "FALSE",
		altsyncram_component.read_during_write_mode_mixed_ports = "DONT_CARE",
		altsyncram_component.widthad_a = 8,
		altsyncram_component.widthad_b = 8,
		altsyncram_component.width_a = pRAM_UNIT_WIDTH,
		altsyncram_component.width_b = pRAM_UNIT_WIDTH,
		altsyncram_component.width_byteena_a = 1;
endmodule
