// SRAM Module
// This will simplify the interface to the SRAM
// simplifying overall complexity of design

// note:  WriteByte is a parameter that allows you to write
// individual bytes to the memory block.  The bit 1 determines
// whether this feature is on, the other bit determines whether it is the
// first byte or the second:
// 11 writes first byte, 10 writes second byte, 0x writes whole word

// DE2 Define
`define SRAM_INTERFACE_DE2 .ioSRAM_DQ(SRAM_DQ),.oSRAM_ADDR(SRAM_ADDR),.oSRAM_UB_N(SRAM_UB_N),.oSRAM_LB_N(SRAM_LB_N),.oSRAM_WE_N(SRAM_WE_N),.oSRAM_CE_N(SRAM_CE_N),.oSRAM_OE_N(SRAM_OE_N)

// Usage template
//module SRAM
//(
//    // User Control
//    .iAddr(<<Address>>), .iWE_N(<<Write Enable>>),
//    .iWriteData(<<Write Data 16'b>>),
//    .oReadData(<<Read Data 16'b>>),
//    // Actual SRAM
//    `SRAM_INTERFACE_DE2
//);

module SRAM
(
    // User Control
    iAddr, iWE_N,
    iWriteData,
    oReadData,
    // Actual SRAM
    ioSRAM_DQ, oSRAM_ADDR,
    oSRAM_UB_N, oSRAM_LB_N,
    oSRAM_WE_N,
    oSRAM_CE_N, oSRAM_OE_N
);

// User Control
input   wire [17:0] iAddr;
input   wire        iWE_N;
input   wire [15:0] iWriteData;
output  wire [15:0] oReadData;

// Actual SRAM
inout	wire [15:0]	ioSRAM_DQ;		//	SRAM Data bus 16 Bits
output	wire [17:0]	oSRAM_ADDR;		//	SRAM Address bus 18 Bits
output  wire	    oSRAM_UB_N;		//	SRAM High-byte Data Mask
output	wire	    oSRAM_LB_N;		//	SRAM Low-byte Data Mask
output	wire	    oSRAM_WE_N;		//	SRAM Write Enable
output	wire	    oSRAM_CE_N;		//	SRAM Chip Enable
output	wire	    oSRAM_OE_N;		//	SRAM Output Enable

// SRAM Stuff
assign ioSRAM_DQ = (iWE_N) ? 16'hzzzz : iWriteData;

assign oSRAM_ADDR = iAddr;
assign oSRAM_UB_N = 0;	    // hi byte select enabled
assign oSRAM_LB_N = 0;		// lo byte select enabled
assign oSRAM_CE_N = 0;		// chip is enabled
assign oSRAM_WE_N = iWE_N;	// Notice WE is negated
assign oSRAM_OE_N = 0;		//output enable is overidden by WE

// The idea is so that SRAM_DQ is not directly manipulated
assign oReadData = ioSRAM_DQ;

endmodule
