`include "SRAM.v"

`define SCREEN_WIDTH 320
`define SCREEN_HEIGHT 240
`define SCREEN_MARGIN 7

// Creates a 640x480 screen buffer
// addressable through x, y coordinates 10 bits wide
// Each memory element contains 2 pixels, each pixel is 8 bits deep
// this means that each color contains 2 bits, or a total of 63 total
// colors, with 2 bits for alpha channel if ever needed.

// Implementation Template:
// module SRAM_SCREEN_BUFFER
// (
//     // User Data
//     .iCoordX( ), .iCoordY( ),
//     .iwe_N( ),
//     .iWriteR( ), .iWriteG( ), .iWriteB( ),
//     .oReadData( ),
//     // Actual SRAM
//     `SRAM_INTERFACE_DE2
// );

module SRAM_SCREEN_BUFFER
(
    // User Data
    iCoordX, iCoordY,
    iwe_N,
    iWriteR, iWriteG, iWriteB,
    oReadData,
    // Actual SRAM
    ioSRAM_DQ, oSRAM_ADDR,
    oSRAM_UB_N, oSRAM_LB_N,
    oSRAM_WE_N,
    oSRAM_CE_N, oSRAM_OE_N
);

// User Data
input   wire [9:0]  iCoordX;
input   wire [9:0]  iCoordY;
input   wire        iwe_N;
input   wire [1:0]  iWriteR;
input   wire [1:0]  iWriteG;
input   wire [1:0]  iWriteB;
output  wire [7:0]  oReadData;
// Actual SRAM
inout	wire [15:0]	ioSRAM_DQ;		//	SRAM Data bus 16 Bits
output	wire [17:0]	oSRAM_ADDR;		//	SRAM Address bus 18 Bits
output  wire	    oSRAM_UB_N;		//	SRAM High-byte Data Mask
output	wire	    oSRAM_LB_N;		//	SRAM Low-byte Data Mask
output	wire	    oSRAM_WE_N;		//	SRAM Write Enable
output	wire	    oSRAM_CE_N;		//	SRAM Chip Enable
output	wire	    oSRAM_OE_N;		//	SRAM Output Enable

wire [17:0] wAddr;
wire [1:0] wWriteByte;
wire [15:0] wDataWrite;
wire [15:0] wReadData;

assign wAddr = {iCoordX[9:1],iCoordY[8:0]};
assign wWriteByte = {1'b1, iCoordX[0]};

assign wDataWrite = {8'hzz, iWriteR, iWriteG, iWriteB, 2'b0};

assign oReadData = (iCoordX[0]) ? wReadData[15:8] : wReadData[7:0];
//assign oReadData = ioSRAM_DQ[7:0];

SRAM sram
(
    // User Control
    .iAddr(wAddr), .iWE_N(iwe_N),
    .iWriteByte(wWriteByte),
    .iWriteData(wDataWrite),
    .oReadData(wReadData),
    // Actual SRAM
    .ioSRAM_DQ(ioSRAM_DQ), .oSRAM_ADDR(oSRAM_ADDR),
    .oSRAM_UB_N(oSRAM_UB_N), .oSRAM_LB_N(oSRAM_LB_N),
    .oSRAM_WE_N(oSRAM_WE_N),
    .oSRAM_CE_N(oSRAM_CE_N), .oSRAM_OE_N(oSRAM_OE_N)

);

endmodule
