# GPUAPI.asm defines the different functions for manipulation of the 
# GPU interface through the in/out interface on the NDMA CPU

# NOTE: Connection to the GPU interface will be through one register input port
# and one register output port with the following conventions:
# Input
# Bit 0 : Complete Flag
#
# Output
# Bit 15-0 : Instruction
# Bit 16 : Valid Flag

# PlotPixel (ppix)
# X - $a0
# Y - $a1
# Color - $a2
plotpixel:
	addi $t0, $0, x0
	add $t1, $t0, $a0
	lui $t2, 1
	add $t1, $t1, $t2
	out $5, $t1
	
ppix_while_x:
	in $t2, $1
	beq $t2, $0, ppix_while_x
	
	out $5, $0
	
	addi $t0, $0, x0400
	add $t1, $t0, $a1
	lui $t2, 1
	add $t1, $t1, $t2
	out $5, $t1
	
ppix_while_y:
	in $t2, $1
	beq $t2, $0, ppix_while_y
	
	out $5, $0
	
	addi $t0, $0, x1C00
	add $t1, $t0, $a2
	lui $t2, 1
	add $t1, $t1, $t2
	out $5, $t1
	
ppix_while_pixel:
	in $t2, $1
	beq $t2, $0, ppix_while_pixel
jr $ra

# Draw Char (dchar)
# X - $a0
# Y - $a1
# Char - $a2
# Color - $a3
drawchar:
	addi $t0, $0, x0
	add $t1, $t0, $a0
	lui $t2, 1
	add $t1, $t1, $t2
	out $5, $t1
	
dchar_while_x:
	in $t2, $1
	beq $t2, $0, dchar_while_x
	
	out $5, $0
	
	addi $t0, $0, x0400
	add $t1, $t0, $a1
	lui $t2, 1
	add $t1, $t1, $t2
	out $5, $t1
	
dchar_while_y:
	in $t2, $1
	beq $t2, $0, dchar_while_y
	
	out $5, $0	
	
	addi $t0, $0, x1000
	add $t1, $t0, $a2
	lui $t2, 1
	add $t1, $t1, $t2
	out $5, $t1

dchar_while_char:
	in $t2, $1
	beq $t2, $0, dchar_while_char
	
	out $5, $0
	
	addi $t0, $0, x1400
	add $t1, $t0, $a3
	lui $t2, 1
	add $t1, $t1, $t2
	out $5, $t1

dchar_while_draw:
	in $t2, $1
	beq $t2, $0, dchar_while_draw
jr $ra

# Wait Char (waitchar)
# Returns 1 if char is waiting
# return 0 if not result in $v0
waitchar:
	in $v0, $2
	srl $v0, $v0, 9
	xori $v0, $v0, 1
jr $ra

# Get Char (getchar)
# Waits until character written to PS2 Buffer
# then saves it into $v0 ($2)
# *char - $a0

getchar:
	in $t1, $2
	srl $t1, $t1, 9
	bne $t1, $0, getchar
	
	outi $6, 1
	
gchar_complete:
	in $t1, $2
	srl $t1, $t1, 8
	andi $t1, $t1, 1
	beq $t1, $0, gchar_complete
	
	in $v0, $2
	andi $v0, $v0, xFF
	
	out $6, $0