`include "../../CPU/CPU.v"
`include "../../MEM/M4K/M4KMEM_DUALPORT.v"
`include "../../MEM/M4K/M4KMEM.v"
`include "../../IO/HexDecode/HexDecode.v"
`include "../../VGA/GPU/GPU.v"
`include "../../IO/RS232/RS232InputBuffer.v"
`include "../../IO/PS2/PS2InputBuffer.v"
`include "../../CPU/RS232BootLoader.v"
`include "ndma_pll.v"

module NDMA_TOP
	(
		////////////////////	Clock Input	 	////////////////////
		CLOCK_27,						//	27 MHz
		CLOCK_50,						//	50 MHz
		EXT_CLOCK,						//	External Clock
		////////////////////	Push Button		////////////////////
		KEY,							//	Pushbutton[3:0]
		////////////////////	DPDT Switch		////////////////////
		SW,								//	Toggle Switch[17:0]
		////////////////////	7-SEG Dispaly	////////////////////
		HEX0,							//	Seven Segment Digit 0
		HEX1,							//	Seven Segment Digit 1
		HEX2,							//	Seven Segment Digit 2
		HEX3,							//	Seven Segment Digit 3
		HEX4,							//	Seven Segment Digit 4
		HEX5,							//	Seven Segment Digit 5
		HEX6,							//	Seven Segment Digit 6
		HEX7,							//	Seven Segment Digit 7
		////////////////////////	LED		////////////////////////
		LEDG,							//	LED Green[8:0]
		LEDR,							//	LED Red[17:0]
		////////////////////////	UART	////////////////////////
		UART_TXD,						//	UART Transmitter
		UART_RXD,						//	UART Receiver
		/*////////////////////////	IRDA	////////////////////////
		IRDA_TXD,						//	IRDA Transmitter
		IRDA_RXD,						//	IRDA Receiver   */
		/////////////////////	SDRAM Interface		////////////////
		DRAM_DQ,						//	SDRAM Data bus 16 Bits
		DRAM_ADDR,						//	SDRAM Address bus 12 Bits
		DRAM_LDQM,						//	SDRAM Low-byte Data Mask
		DRAM_UDQM,						//	SDRAM High-byte Data Mask
		DRAM_WE_N,						//	SDRAM Write Enable
		DRAM_CAS_N,						//	SDRAM Column Address Strobe
		DRAM_RAS_N,						//	SDRAM Row Address Strobe
		DRAM_CS_N,						//	SDRAM Chip Select
		DRAM_BA_0,						//	SDRAM Bank Address 0
		DRAM_BA_1,						//	SDRAM Bank Address 0
		DRAM_CLK,						//	SDRAM Clock
		DRAM_CKE,						//	SDRAM Clock Enable
		////////////////////	Flash Interface		////////////////
		FL_DQ,							//	FLASH Data bus 8 Bits
		FL_ADDR,						//	FLASH Address bus 22 Bits
		FL_WE_N,						//	FLASH Write Enable
		FL_RST_N,						//	FLASH Reset
		FL_OE_N,						//	FLASH Output Enable
		FL_CE_N,						//	FLASH Chip Enable
		////////////////////	SRAM Interface		////////////////
		SRAM_DQ,						//	SRAM Data bus 16 Bits
		SRAM_ADDR,						//	SRAM Address bus 18 Bits
		SRAM_UB_N,						//	SRAM High-byte Data Mask
		SRAM_LB_N,						//	SRAM Low-byte Data Mask
		SRAM_WE_N,						//	SRAM Write Enable
		SRAM_CE_N,						//	SRAM Chip Enable
		SRAM_OE_N,						//	SRAM Output Enable
		////////////////////	ISP1362 Interface	////////////////
		OTG_DATA,						//	ISP1362 Data bus 16 Bits
		OTG_ADDR,						//	ISP1362 Address 2 Bits
		OTG_CS_N,						//	ISP1362 Chip Select
		OTG_RD_N,						//	ISP1362 Write
		OTG_WR_N,						//	ISP1362 Read
		OTG_RST_N,						//	ISP1362 Reset
		OTG_FSPEED,						//	USB Full Speed,	0 = Enable, Z = Disable
		OTG_LSPEED,						//	USB Low Speed, 	0 = Enable, Z = Disable
		OTG_INT0,						//	ISP1362 Interrupt 0
		OTG_INT1,						//	ISP1362 Interrupt 1
		OTG_DREQ0,						//	ISP1362 DMA Request 0
		OTG_DREQ1,						//	ISP1362 DMA Request 1
		OTG_DACK0_N,					//	ISP1362 DMA Acknowledge 0
		OTG_DACK1_N,					//	ISP1362 DMA Acknowledge 1
		////////////////////	LCD Module 16X2		////////////////
		LCD_ON,							//	LCD Power ON/OFF
		LCD_BLON,						//	LCD Back Light ON/OFF
		LCD_RW,							//	LCD Read/Write Select, 0 = Write, 1 = Read
		LCD_EN,							//	LCD Enable
		LCD_RS,							//	LCD Command/Data Select, 0 = Command, 1 = Data
		LCD_DATA,						//	LCD Data bus 8 bits
		////////////////////	SD_Card Interface	////////////////
		SD_DAT,							//	SD Card Data
		SD_DAT3,						//	SD Card Data 3
		SD_CMD,							//	SD Card Command Signal
		SD_CLK,							//	SD Card Clock
		////////////////////	USB JTAG link	////////////////////
		TDI,  							// CPLD -> FPGA (data in)
		TCK,  							// CPLD -> FPGA (clk)
		TCS,  							// CPLD -> FPGA (CS)
	    TDO,  							// FPGA -> CPLD (data out)
		////////////////////	I2C		////////////////////////////
		I2C_SDAT,						//	I2C Data
		I2C_SCLK,						//	I2C Clock
		////////////////////	PS2		////////////////////////////
		PS2_DAT,						//	PS2 Data
		PS2_CLK,						//	PS2 Clock
		////////////////////	VGA		////////////////////////////
		VGA_CLK,   						//	VGA Clock
		VGA_HS,							//	VGA H_SYNC
		VGA_VS,							//	VGA V_SYNC
		VGA_BLANK,						//	VGA BLANK
		VGA_SYNC,						//	VGA SYNC
		VGA_R,   						//	VGA Red[9:0]
		VGA_G,	 						//	VGA Green[9:0]
		VGA_B,  						//	VGA Blue[9:0]
		////////////	Ethernet Interface	////////////////////////
		ENET_DATA,						//	DM9000A DATA bus 16Bits
		ENET_CMD,						//	DM9000A Command/Data Select, 0 = Command, 1 = Data
		ENET_CS_N,						//	DM9000A Chip Select
		ENET_WR_N,						//	DM9000A Write
		ENET_RD_N,						//	DM9000A Read
		ENET_RST_N,						//	DM9000A Reset
		ENET_INT,						//	DM9000A Interrupt
		ENET_CLK,						//	DM9000A Clock 25 MHz
		////////////////	Audio CODEC		////////////////////////
		AUD_ADCLRCK,					//	Audio CODEC ADC LR Clock
		AUD_ADCDAT,						//	Audio CODEC ADC Data
		AUD_DACLRCK,					//	Audio CODEC DAC LR Clock
		AUD_DACDAT,						//	Audio CODEC DAC Data
		AUD_BCLK,						//	Audio CODEC Bit-Stream Clock
		AUD_XCK,						//	Audio CODEC Chip Clock
		////////////////	TV Decoder		////////////////////////
		TD_DATA,    					//	TV Decoder Data bus 8 bits
		TD_HS,							//	TV Decoder H_SYNC
		TD_VS,							//	TV Decoder V_SYNC
		TD_RESET,						//	TV Decoder Reset
		////////////////////	GPIO	////////////////////////////
		GPIO_0,							//	GPIO Connection 0
		GPIO_1							//	GPIO Connection 1
	);

////////////////////////	Clock Input	 	////////////////////////
input			CLOCK_27;				//	27 MHz
input			CLOCK_50;				//	50 MHz
input			EXT_CLOCK;				//	External Clock
////////////////////////	Push Button		////////////////////////
input	[3:0]	KEY;					//	Pushbutton[3:0]
////////////////////////	DPDT Switch		////////////////////////
input	[17:0]	SW;						//	Toggle Switch[17:0]
////////////////////////	7-SEG Dispaly	////////////////////////
output	[6:0]	HEX0;					//	Seven Segment Digit 0
output	[6:0]	HEX1;					//	Seven Segment Digit 1
output	[6:0]	HEX2;					//	Seven Segment Digit 2
output	[6:0]	HEX3;					//	Seven Segment Digit 3
output	[6:0]	HEX4;					//	Seven Segment Digit 4
output	[6:0]	HEX5;					//	Seven Segment Digit 5
output	[6:0]	HEX6;					//	Seven Segment Digit 6
output	[6:0]	HEX7;					//	Seven Segment Digit 7
////////////////////////////	LED		////////////////////////////
output	[8:0]	LEDG;					//	LED Green[8:0]
output	[17:0]	LEDR;					//	LED Red[17:0]
////////////////////////////	UART	////////////////////////////
output			UART_TXD;				//	UART Transmitter
input			UART_RXD;				//	UART Receiver
/*////////////////////////////	IRDA	////////////////////////////
output			IRDA_TXD;				//	IRDA Transmitter
input			IRDA_RXD;				//	IRDA Receiver*/
///////////////////////		SDRAM Interface	////////////////////////
inout	[15:0]	DRAM_DQ;				//	SDRAM Data bus 16 Bits
output	[11:0]	DRAM_ADDR;				//	SDRAM Address bus 12 Bits
output			DRAM_LDQM;				//	SDRAM Low-byte Data Mask
output			DRAM_UDQM;				//	SDRAM High-byte Data Mask
output			DRAM_WE_N;				//	SDRAM Write Enable
output			DRAM_CAS_N;				//	SDRAM Column Address Strobe
output			DRAM_RAS_N;				//	SDRAM Row Address Strobe
output			DRAM_CS_N;				//	SDRAM Chip Select
output			DRAM_BA_0;				//	SDRAM Bank Address 0
output			DRAM_BA_1;				//	SDRAM Bank Address 0
output			DRAM_CLK;				//	SDRAM Clock
output			DRAM_CKE;				//	SDRAM Clock Enable
////////////////////////	Flash Interface	////////////////////////
inout	[7:0]	FL_DQ;					//	FLASH Data bus 8 Bits
output	[21:0]	FL_ADDR;				//	FLASH Address bus 22 Bits
output			FL_WE_N;				//	FLASH Write Enable
output			FL_RST_N;				//	FLASH Reset
output			FL_OE_N;				//	FLASH Output Enable
output			FL_CE_N;				//	FLASH Chip Enable
////////////////////////	SRAM Interface	////////////////////////
inout	[15:0]	SRAM_DQ;				//	SRAM Data bus 16 Bits
output	[17:0]	SRAM_ADDR;				//	SRAM Address bus 18 Bits
output			SRAM_UB_N;				//	SRAM High-byte Data Mask
output			SRAM_LB_N;				//	SRAM Low-byte Data Mask
output			SRAM_WE_N;				//	SRAM Write Enable
output			SRAM_CE_N;				//	SRAM Chip Enable
output			SRAM_OE_N;				//	SRAM Output Enable
////////////////////	ISP1362 Interface	////////////////////////
inout	[15:0]	OTG_DATA;				//	ISP1362 Data bus 16 Bits
output	[1:0]	OTG_ADDR;				//	ISP1362 Address 2 Bits
output			OTG_CS_N;				//	ISP1362 Chip Select
output			OTG_RD_N;				//	ISP1362 Write
output			OTG_WR_N;				//	ISP1362 Read
output			OTG_RST_N;				//	ISP1362 Reset
output			OTG_FSPEED;				//	USB Full Speed,	0 = Enable, Z = Disable
output			OTG_LSPEED;				//	USB Low Speed, 	0 = Enable, Z = Disable
input			OTG_INT0;				//	ISP1362 Interrupt 0
input			OTG_INT1;				//	ISP1362 Interrupt 1
input			OTG_DREQ0;				//	ISP1362 DMA Request 0
input			OTG_DREQ1;				//	ISP1362 DMA Request 1
output			OTG_DACK0_N;			//	ISP1362 DMA Acknowledge 0
output			OTG_DACK1_N;			//	ISP1362 DMA Acknowledge 1
////////////////////	LCD Module 16X2	////////////////////////////
inout	[7:0]	LCD_DATA;				//	LCD Data bus 8 bits
output			LCD_ON;					//	LCD Power ON/OFF
output			LCD_BLON;				//	LCD Back Light ON/OFF
output			LCD_RW;					//	LCD Read/Write Select, 0 = Write, 1 = Read
output			LCD_EN;					//	LCD Enable
output			LCD_RS;					//	LCD Command/Data Select, 0 = Command, 1 = Data
////////////////////	SD Card Interface	////////////////////////
inout			SD_DAT;					//	SD Card Data
inout			SD_DAT3;				//	SD Card Data 3
inout			SD_CMD;					//	SD Card Command Signal
output			SD_CLK;					//	SD Card Clock
////////////////////////	I2C		////////////////////////////////
inout			I2C_SDAT;				//	I2C Data
output			I2C_SCLK;				//	I2C Clock
////////////////////////	PS2		////////////////////////////////
input		 	PS2_DAT;				//	PS2 Data
input			PS2_CLK;				//	PS2 Clock
////////////////////	USB JTAG link	////////////////////////////
input  			TDI;					// CPLD -> FPGA (data in)
input  			TCK;					// CPLD -> FPGA (clk)
input  			TCS;					// CPLD -> FPGA (CS)
output 			TDO;					// FPGA -> CPLD (data out)
////////////////////////	VGA			////////////////////////////
output			VGA_CLK;   				//	VGA Clock
output			VGA_HS;					//	VGA H_SYNC
output			VGA_VS;					//	VGA V_SYNC
output			VGA_BLANK;				//	VGA BLANK
output			VGA_SYNC;				//	VGA SYNC
output	[9:0]	VGA_R;   				//	VGA Red[9:0]
output	[9:0]	VGA_G;	 				//	VGA Green[9:0]
output	[9:0]	VGA_B;   				//	VGA Blue[9:0]
////////////////	Ethernet Interface	////////////////////////////
inout	[15:0]	ENET_DATA;				//	DM9000A DATA bus 16Bits
output			ENET_CMD;				//	DM9000A Command/Data Select, 0 = Command, 1 = Data
output			ENET_CS_N;				//	DM9000A Chip Select
output			ENET_WR_N;				//	DM9000A Write
output			ENET_RD_N;				//	DM9000A Read
output			ENET_RST_N;				//	DM9000A Reset
input			ENET_INT;				//	DM9000A Interrupt
output			ENET_CLK;				//	DM9000A Clock 25 MHz
////////////////////	Audio CODEC		////////////////////////////
inout			AUD_ADCLRCK;			//	Audio CODEC ADC LR Clock
input			AUD_ADCDAT;				//	Audio CODEC ADC Data
inout			AUD_DACLRCK;			//	Audio CODEC DAC LR Clock
output			AUD_DACDAT;				//	Audio CODEC DAC Data
inout			AUD_BCLK;				//	Audio CODEC Bit-Stream Clock
output			AUD_XCK;				//	Audio CODEC Chip Clock
////////////////////	TV Devoder		////////////////////////////
input	[7:0]	TD_DATA;    			//	TV Decoder Data bus 8 bits
input			TD_HS;					//	TV Decoder H_SYNC
input			TD_VS;					//	TV Decoder V_SYNC
output			TD_RESET;				//	TV Decoder Reset
////////////////////////	GPIO	////////////////////////////////
inout	[35:0]	GPIO_0;					//	GPIO Connection 0
inout	[35:0]	GPIO_1;					//	GPIO Connection 1

//	All inout port turn to tri-state
assign	DRAM_DQ		=	16'hzzzz;
assign	FL_DQ		=	8'hzz;
//assign	SRAM_DQ		=	16'hzzzz;
assign	OTG_DATA	=	16'hzzzz;
assign	LCD_DATA	=	8'hzz;
assign	SD_DAT		=	1'bz;
assign	I2C_SDAT	=	1'bz;
assign	ENET_DATA	=	16'hzzzz;
//assign	AUD_ADCLRCK	=	1'bz;
//assign	AUD_DACLRCK	=	1'bz;
//assign	AUD_BCLK	=	1'bz;
//assign	GPIO_0		=	36'hzzzzzzzzz;
//assign	GPIO_1		=	36'hzzzzzzzzz;

// Master Control Signals
wire            wMRST = ~KEY[0];
wire            wMCLK;
reg             rMCLK = 0;
reg [3:0]       rMCLK_cnt = 0;

// Try to seperate the CPU clock from the VGA_PLL
always@(posedge CLOCK_27) begin
    if(rMCLK_cnt < 9)
        begin
            rMCLK_cnt <= rMCLK_cnt + 1;
        end
    else
        begin
            rMCLK_cnt <= 0;
            rMCLK <= ~rMCLK;
        end
end

// This will allow switching between single stepped operation
// and full speed operation.
wire            wCLK_SWITCH = (SW[0]) ? rMCLK : ~KEY[1];

// GPU Interface
wire            wGPUComplete;
wire            wGPUValid;
wire    [15:0]  wGPUInstruction;

// PS2 Input Buffer
wire            wPS2BufferEmpty;
wire    [7:0]   wPS2ASCIIOut;
wire            wPS2Complete;
wire            wPS2Valid;

// START CPU 1
wire    [31:0]  PC1, Imem1, dAddr1, Din1, Dout1;
wire            dWrite1;
// CPU 1 Network
wire    [31:0]  nBI1, sBI1, wBI1, eBI1;
wire    [31:0]  nBO1, sBO1, wBO1, eBO1;
// IO
wire    [31:0]  wPortA1, wPortB1, wPortC1, wPortD1;
wire    [31:0]  wPortE1, wPortF1, wPortG1, wPortH1;

// Boot Loader
wire    [7:0]   wASCII_TX;
wire            wValid_TX;
wire            wComplete_TX;

wire    [7:0]   wASCII_RX;
wire            wBufferEmpty;
wire            wValid_RX;
wire            wComplete_RX;

assign LEDG[0] = wBufferEmpty;
assign LEDG[7] = wBootLoad;
//assign LEDR = wMem1Addr;

wire    [31:0]  wPC;
wire    [31:0]  wIns;
wire            wBootLoad;
wire            wWE_N;
wire    [31:0]  wMem1Addr = (wBootLoad) ? wPC : PC1;

wire            wBootCLK;

//ndma_pll ndmapll(CLOCK_50, wBootCLK);

// CPU 1 BootLoader
RS232InputBuffer rs232
(
    .iCLK(CLOCK_50), .iMRST(wMRST),
    // RS-232 Connections
   `UART_INTERFACE_DE2,
    // Buffer Interface
    // Transmit
    .iASCII_TX(wASCII_TX),
    .iValid_TX(wValid_TX),
    .oComplete_TX(wComplete_TX),
    // Receive
    .iValid(wValid_RX),
    .oBufferEmpty(wBufferEmpty),
    .oASCIIOut(wASCII_RX),
    .oComplete(wComplete_RX)
);

RS232BootLoader bootload
(
    .iCLK(CLOCK_50), .iMRST(wMRST),
    // Transmit
    .oASCII(wASCII_TX),             // Transmit ASCII Value
    .oValid_TX(wValid_TX),          // Valid Transmit
    .iComplete_TX(wComplete_TX),    // Transmit Complete
    // Recieve
    .iASCII(wASCII_RX),             // Receive ASCII Value
    .iBufferEmpty(wBufferEmpty),    // Buffer Empty Value
    .oValid_RX(wValid_RX),          // Valid Receive
    .iComplete_RX(wComplete_RX),    // Receive Complete
    //.oState(LEDR),
    .oBootLoad(wBootLoad),
    .oPC(wPC),
    .oIns(wIns),
    .oWE_N(wWE_N)
);

// Instruction Memory CPU 1
M4KMEM m4kmem_ins1(
    .iCLK(CLOCK_50), .iMRST(~wMRST),
    .iWE_N(wWE_N), .iAddr(wMem1Addr),
    .iWriteData(wIns), .oReadData(Imem1)
);
// m4kmem parameters
defparam m4kmem_ins1.pRAM_ADDR_WIDTH = 8,
         m4kmem_ins1.pRAM_UNIT_COUNT = 256,
         m4kmem_ins1.pRAM_UNIT_WIDTH = 32,
         m4kmem_ins1.pINIT_FILE = "memory.mif";
// end m4kmem parameters

HexDecode hex0(Imem1[3:0], HEX0);
HexDecode hex1(Imem1[7:4], HEX1);
HexDecode hex2(Imem1[11:8], HEX2);
HexDecode hex3(Imem1[15:12], HEX3);
HexDecode hex4(Imem1[19:16], HEX4);
HexDecode hex5(Imem1[23:20], HEX5);
HexDecode hex6(Imem1[27:24], HEX6);
HexDecode hex7(Imem1[31:28], HEX7);

// Data Memory CPU 1
// negative edge triggered ?
M4KMEM m4kmem_data1(
    .iCLK(CLOCK_50), .iMRST(~wMRST),
    .iWE_N(~dWrite1), .iAddr(dAddr1),
    .iWriteData(Din1), .oReadData(Dout1)
);
// m4kmem parameters
defparam m4kmem_data1.pRAM_ADDR_WIDTH = 8,
         m4kmem_data1.pRAM_UNIT_COUNT = 256,
         m4kmem_data1.pRAM_UNIT_WIDTH = 32,
         m4kmem_data1.pINIT_FILE = "data.mif";
// end m4kmem parameters

// NDMA CPU
CPU cpu1
(
	.CLK 		(wCLK_SWITCH),
	.MRST 		(wMRST | wBootLoad),        // wBootLoad resets CPU (PC -> 0)
	.oIMemAddr	(PC1),
	.iMemIn		(Imem1),
	.dMemIn		(Dout1),
	.dAddr		(dAddr1),
	.dWrite		(dWrite1),
	.WD			(Din1),
    // Network Side (dont need for this test)
	.nBusIn		(nBI1), .nBusOut	(nBO1),
	.sBusIn		(nBO3), .sBusOut	(sBO1),
	.eBusIn		(wBO2), .eBusOut	(eBO1),
	.wBusIn		(nBI1), .wBusOut	(wBO1),
    // IO Side
    .iPortA(wPortA1),   // Input From GPU
    .oPortE(wPortE1),   // Output to GPU
    .iPortB(wPortB1),   // Input from PS2 Buffer
    .oPortF(wPortF1)    // Output to PS2 buffer
);
// END CPU 1

assign LEDR = PC1;

// PS2 Input Buffer Belongs to CPU1
// ***********************************************
assign wPortB1 = {wPS2BufferEmpty, wPS2Complete, wPS2ASCIIOut};
assign wPS2Valid = wPortF1[0];
assign LEDG[6] = wPS2Valid;

PS2InputBuffer ps2buf
(
    .iCLK(CLOCK_50), .iMRST(wMRST | wBootLoad),
    `PS2_INTERFACE_DE2,
    .iValid(wPS2Valid),
    .oBufferEmpty(wPS2BufferEmpty),
    .oASCIIOut(wPS2ASCIIOut),
    .oComplete(wPS2Complete)
);
// **********************************************

// GPU belongs to CPU1
// **********************************************
assign wPortA1 = {15'd0, wGPUComplete};
assign wGPUInstruction = wPortE1[15:0];
assign wGPUValid = wPortE1[16];

GPU gpu
(
    `VGA_CLOCKS_DE2,
    // User Data
    .iReset(wMRST | wBootLoad),
    .iValid(wGPUValid),
    .iIns(wGPUInstruction),
    .oComplete(wGPUComplete),
    .oVGA_CTRL_CLK(wVGA_CTRL_CLK),
    //.oAUDCtrlClk(wAUDCtrlClk),
    //.oDLY_RST(wDLY_RST),
    `VGA_INTERFACE_DE2,
    `VGA_TD_RESET_DE2,
    // SRAM Interface
    `SRAM_INTERFACE_DE2
);
// ***********************************************

endmodule
