// ASCIIVGA module
// Will print a character to the screen
// follows similar conventions regarding iValidChar and oCharComplete
// as the Bresenham module.  The iValidChar goes high when the inputs
// are valid.  The CharComplete will go high when the char is printed
// to the screen and only then may the caller lower the iValidChar
// signal.

`include "ASCIIROM.v"

module ASCIIVGA
(
    iLock, iCLK,
    // User Inputs
    iValidChar,
    iX1, iY1,
    iChar,
    iColor,
    // Output to signify completion
    oCharComplete,
    // Screen Buffer Hookup
    oXCoord, oYCoord,
    oWE_N,
    oWriteR, oWriteG, oWriteB
);

input   wire            iLock;  // Halt's plotting
input   wire            iCLK;
input   wire            iValidChar;

input   wire    [9:0]   iX1;
input   wire    [9:0]   iY1;

input   wire    [7:0]   iChar;
input   wire    [7:0]   iColor;

output  reg     [9:0]   oXCoord;
output  reg     [9:0]   oYCoord;
output  reg             oWE_N = 1'b1;
output  wire    [1:0]   oWriteR, oWriteG, oWriteB;
output  reg             oCharComplete = 0;

// State Variables
reg     [3:0]   rState = Init;

reg     [9:0]   rXCoord;
reg     [9:0]   rYCoord;
reg     [9:0]   rX1;//, rY1;
reg             rLock;

wire    [39:0]  wCharOut;
reg     [39:0]  rCharOut;

reg     [5:0]   rCharCount;
reg     [2:0]   rLineCount;

assign oWriteR = iColor[7:6];
assign oWriteG = iColor[5:4];
assign oWriteB = iColor[3:2];

ASCIIROM asciirom
(
    .iCLK(iCLK),
    .iAddress(iChar),
    .oCharBM(wCharOut)
);

parameter Init = 4'd0,
          DoneState = 4'd8;

//always@(iValidLine or iX1 or iX2 or iY1 or iY2 or rState) begin
always@(posedge iCLK) begin
    if(~iValidChar)
        begin
            rState <= Init;
            oWE_N <= 1'b1;
            oCharComplete <= 0;
            rCharCount <= 39;
            rLineCount <= 0;
        end
    else
    if(iLock)
        begin
            case(rState)
                Init:
                    begin
                        oXCoord <= iX1;
                        oYCoord <= iY1;
                        rX1 <= iX1;
                        //rY1 <= iY1;
                        rCharOut <= wCharOut;
                        oWE_N <= ~wCharOut[rCharCount];
                        rState <= 1;
                    end

                1:
                    begin
                        oWE_N <= 1'b1;
                        if(rCharCount != 0) // last bit doesnt count
                            begin
                                if(rLock)
                                    begin
                                        if(rLineCount == 4)
                                            begin
                                                oYCoord <= oYCoord + 1;
                                                oXCoord <= rX1;
                                                rLineCount <= 0;
                                            end
                                        else
                                            begin
                                                oXCoord <= oXCoord + 1;
                                                rLineCount <= rLineCount + 1;
                                            end
                                        rCharCount <= rCharCount - 1;
                                    end
                                rState <= 2;
                            end
                        else
                            begin
                                rState <= DoneState;
                            end
                    end

                2:
                    begin
                        oWE_N <= ~wCharOut[rCharCount];
                        rState <= 3;
                    end
                3:
                    begin
                        oWE_N <= 1'b1;
                        rLock <= 1;
                        rState <= 1;
                    end

                DoneState:
                    begin
                        oCharComplete <= 1;
                        oWE_N <= 1'b1;
                        rCharCount <= 15;
                        rLineCount <= 0;
                    end
            endcase
        end else
            begin
                rLock <= 0;
                oWE_N <= 1'b1;
            end
end


endmodule
