// PixelPlot Module
// Will plot a pixel of a specific color to the screen
// Follows similar conventions regarding iValidPixel and oPixelComplete
// as the Bresenham module.

module PixelPlot
(
    iLock, iCLK,
    // User Inputs
    iValidPixel,
    iX1, iY1,
    iColor,
    // Output to signify completion
    oPixelComplete,
    // Screen Buffer Hookup
    oXCoord, oYCoord,
    oWE_N,
    oWriteR, oWriteG, oWriteB
);

input   wire            iLock;  // Halt's plotting
input   wire            iCLK;
input   wire            iValidPixel;

input   wire    [9:0]   iX1;
input   wire    [9:0]   iY1;

input   wire    [7:0]   iColor;

output  reg     [9:0]   oXCoord;
output  reg     [9:0]   oYCoord;
output  reg             oWE_N = 1'b1;
output  wire    [1:0]   oWriteR, oWriteG, oWriteB;
output  reg             oPixelComplete = 0;

// State Variables
reg     [3:0]   rState = Init;
reg             rLock;

assign oWriteR = iColor[7:6];
assign oWriteG = iColor[5:4];
assign oWriteB = iColor[3:2];

parameter Init = 4'd0,
          DoneState = 4'd8;

//always@(iValidLine or iX1 or iX2 or iY1 or iY2 or rState) begin
always@(posedge iCLK) begin
    if(~iValidPixel)
        begin
            rState <= Init;
            oWE_N <= 1'b1;
            oPixelComplete <= 0;
        end
    else
    if(iLock)
        begin
            case(rState)
                Init:
                    begin
                            oXCoord <= iX1;
                            oYCoord <= iY1;
                            oWE_N <= 1'b0;
                            rState <= 1;
                    end

                1:
                    begin
                        oWE_N <= 1'b1;
                        rState <= DoneState;
                    end

                DoneState:
                    begin
                        oPixelComplete <= 1;
                        oWE_N <= 1'b1;
                    end
            endcase
        end else
            begin
                rLock <= 0;
                oWE_N <= 1'b1;
            end
end


endmodule
