`include "./VGA_Controller/Reset_Delay.v"
`include "./VGA_Controller/VGA_Audio_PLL.v"
`include "./VGA_Controller/VGA_Controller.v"

// These defines are helpful for easier code
`define VGA_CLOCKS_DE2 .iCLK50 (CLOCK_50),.iCLK27(CLOCK_27)
`define VGA_INTERFACE_DE2  .oVGA_CLK(VGA_CLK),.oVGA_HS(VGA_HS),.oVGA_VS(VGA_VS),.oVGA_BLANK(VGA_BLANK),.oVGA_SYNC(VGA_SYNC),.oVGA_R(VGA_R),.oVGA_G(VGA_G),.oVGA_B(VGA_B)
`define VGA_TD_RESET_DE2 .oTD_RESET(TD_RESET)

// Instantiating this module will provide an easy interface to the VGA
// this is a lot more straightforward than instantiating all of the
// different modules every single time

// Usage Template:
// VGACNT vgacnt
//(
//    //Clocks
//    `VGA_CLOCKS_DE2,
//    // Host Side
//    .oVGA_ADDR(<<*VGA ADDR OUT*>>),
//    // Data Side
//    .iVGA_R(<<*R DATA*>>), .iVGA_G(<<*G DATA*>>), .iVGA_B(<<*B DATA*>>),
//    .oVGA_X(<<*X OUT*>>), .oVGA_Y(<<*Y OUT*>>),.oVGA_CTRL_CLK(wVGA_CTRL_CLK),
//    // VGA Side
//    `VGA_INTERFACE_DE2,
//    // Bypress 27 MHz
//    `VGA_TD_RESET_DE2
//);


module VGACNT
(
    // Clocks
    iCLK50, iCLK27,
    // Host Side
    oVGA_ADDR,
    // Data Side
    iVGA_R, iVGA_G, iVGA_B,
    oVGA_X, oVGA_Y, oVGA_CTRL_CLK,
    oAUDCtrlClk,
    oDLY_RST,
    // VGA Side
    oVGA_CLK,
    oVGA_HS, oVGA_VS,
    oVGA_BLANK, oVGA_SYNC,
    oVGA_R, oVGA_G, oVGA_B,
    // Bypress 27 Mhz
    oTD_RESET
);
// Clocks
input wire iCLK50;
input wire iCLK27;
// Host Side
output wire[19:0] oVGA_ADDR;
// Data Side
input wire[9:0] iVGA_R;
input wire[9:0] iVGA_G;
input wire[9:0] iVGA_B;
output wire[9:0] oVGA_X;
output wire[9:0] oVGA_Y;
// VGA Side
output wire oVGA_CLK;
output wire oVGA_CTRL_CLK;
output wire oAUDCtrlClk;
output wire oDLY_RST = wDLY_RST;
output wire oVGA_HS;
output wire oVGA_VS;
output wire oVGA_BLANK;
output wire oVGA_SYNC;
output wire[9:0] oVGA_R;
output wire[9:0] oVGA_G;
output wire[9:0] oVGA_B;
// Bypress 27 MHz
output wire oTD_RESET;

// VGA control lines
wire		wAUD_CTRL_CLK;

// Reset Delay
wire wDLY_RST;

assign oTD_RESET = 1'b1;

Reset_Delay	r0
(
	.iCLK	(iCLK50),
	.oRESET	(wDLY_RST)
);

VGA_Audio_PLL p1
(
	.areset	(~wDLY_RST),
	.inclk0	(iCLK27),
	.c0		(oVGA_CTRL_CLK),
	.c1		(oAUDCtrlClk),
	.c2		(oVGA_CLK)
);

VGA_Controller u1(
	//	Host Side
	.iCursor_RGB_EN	(4'b0111),
	.oAddress		(oVGA_ADDR),
	// xy stuff
	.iRed		(iVGA_R),
	.iGreen		(iVGA_G),
	.iBlue		(iVGA_B),
	.oCoord_X	(oVGA_X),
	.oCoord_Y	(oVGA_Y),
	//	VGA Side
	.oVGA_R			(oVGA_R),
	.oVGA_G			(oVGA_G),
	.oVGA_B			(oVGA_B),
	.oVGA_H_SYNC	(oVGA_HS),
	.oVGA_V_SYNC	(oVGA_VS),
	.oVGA_SYNC		(oVGA_SYNC),
	.oVGA_BLANK		(oVGA_BLANK),
	//	Control Signal
	.iCLK			(oVGA_CTRL_CLK),
	.iRST_N			(wDLY_RST)
);

endmodule
